"""
    :codeauthor: :email:`Shane Lee <slee@saltstack.com>`
"""

import errno
import textwrap

import salt.grains.iscsi as iscsi
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase


class IscsiGrainsTestCase(TestCase):
    """
    Test cases for iscsi grains
    """

    def test_windows_iscsi_iqn_grains(self):
        cmd_run_mock = MagicMock(
            return_value={
                "stdout": "iSCSINodeName\niqn.1991-05.com.microsoft:simon-x1\n"
            }
        )
        _grains = {}
        with patch("salt.utils.path.which", MagicMock(return_value=True)):
            with patch("salt.modules.cmdmod.run_all", cmd_run_mock):
                _grains["iscsi_iqn"] = iscsi._windows_iqn()

        self.assertEqual(
            _grains.get("iscsi_iqn"), ["iqn.1991-05.com.microsoft:simon-x1"]
        )

    def test_aix_iscsi_iqn_grains(self):
        cmd_run_mock = MagicMock(
            return_value="initiator_name iqn.localhost.hostid.7f000001"
        )

        _grains = {}
        with patch("salt.modules.cmdmod.run", cmd_run_mock):
            _grains["iscsi_iqn"] = iscsi._aix_iqn()

        self.assertEqual(_grains.get("iscsi_iqn"), ["iqn.localhost.hostid.7f000001"])

    def test_linux_iscsi_iqn_grains(self):
        _iscsi_file = textwrap.dedent(
            """\
            ## DO NOT EDIT OR REMOVE THIS FILE!
            ## If you remove this file, the iSCSI daemon will not start.
            ## If you change the InitiatorName, existing access control lists
            ## may reject this initiator.  The InitiatorName must be unique
            ## for each iSCSI initiator.  Do NOT duplicate iSCSI InitiatorNames.
            InitiatorName=iqn.1993-08.org.debian:01:d12f7aba36
            """
        )

        with patch("salt.utils.files.fopen", mock_open(read_data=_iscsi_file)):
            iqn = iscsi._linux_iqn()

        assert isinstance(iqn, list)
        assert len(iqn) == 1
        assert iqn == ["iqn.1993-08.org.debian:01:d12f7aba36"]

    @patch(
        "salt.utils.files.fopen",
        MagicMock(
            side_effect=IOError(errno.EPERM, "The cables are not the same length.")
        ),
    )
    @patch("salt.grains.iscsi.log", MagicMock())
    def test_linux_iqn_non_root(self):
        """
        Test if linux_iqn is running on salt-master as non-root
        and handling access denial properly.
        :return:
        """
        assert iscsi._linux_iqn() == []
        iscsi.log.debug.assert_called()
        assert "Error while accessing" in iscsi.log.debug.call_args[0][0]
        assert "cables are not the same" in iscsi.log.debug.call_args[0][2].strerror
        assert iscsi.log.debug.call_args[0][2].errno == errno.EPERM
        assert iscsi.log.debug.call_args[0][1] == "/etc/iscsi/initiatorname.iscsi"

    @patch("salt.utils.files.fopen", MagicMock(side_effect=IOError(errno.ENOENT, "")))
    @patch("salt.grains.iscsi.log", MagicMock())
    def test_linux_iqn_no_iscsii_initiator(self):
        """
        Test if linux_iqn is running on salt-master as root.
        iscsii initiator is not there accessible or is not supported.
        :return:
        """
        assert iscsi._linux_iqn() == []
        iscsi.log.debug.assert_not_called()



import logging
import random
import string
from copy import deepcopy

import salt.loader
import salt.modules.boto_s3_bucket as boto_s3_bucket
from salt.utils.versions import LooseVersion
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error,no-name-in-module,unused-import
try:
    import boto
    import boto3
    from botocore.exceptions import ClientError

    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False

# pylint: enable=import-error,no-name-in-module,unused-import

# the boto_s3_bucket module relies on the connect_to_region() method
# which was added in boto 2.8.0
# https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
required_boto3_version = "1.2.1"

log = logging.getLogger(__name__)


def _has_required_boto():
    """
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    """
    if not HAS_BOTO:
        return False
    elif LooseVersion(boto3.__version__) < LooseVersion(required_boto3_version):
        return False
    else:
        return True


if _has_required_boto():
    region = "us-east-1"
    access_key = "GKTADJGHEIQSXMKKRBJ08H"
    secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
    conn_parameters = {
        "region": region,
        "key": access_key,
        "keyid": secret_key,
        "profile": {},
    }
    error_message = (
        "An error occurred (101) when calling the {0} operation: Test-defined error"
    )
    e404_error = ClientError(
        {"Error": {"Code": "404", "Message": "Test-defined error"}}, "msg"
    )
    not_found_error = ClientError(
        {"Error": {"Code": "NoSuchBucket", "Message": "Test-defined error"}}, "msg"
    )
    error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
    create_ret = {
        "Location": "nowhere",
    }
    list_ret = {
        "Buckets": [{"Name": "mybucket", "CreationDate": None}],
        "Owner": {"DisplayName": "testuser", "ID": "12341234123"},
        "ResponseMetadata": {"Key": "Value"},
    }
    config_ret = {
        "get_bucket_acl": {
            "Grants": [
                {
                    "Grantee": {
                        "DisplayName": "testowner",
                        "ID": "sdfghjklqwertyuiopzxcvbnm",
                    },
                    "Permission": "FULL_CONTROL",
                },
                {
                    "Grantee": {
                        "URI": "http://acs.amazonaws.com/groups/global/AllUsers"
                    },
                    "Permission": "READ",
                },
            ],
            "Owner": {"DisplayName": "testowner", "ID": "sdfghjklqwertyuiopzxcvbnm"},
        },
        "get_bucket_cors": {
            "CORSRules": [{"AllowedMethods": ["GET"], "AllowedOrigins": ["*"]}]
        },
        "get_bucket_lifecycle_configuration": {
            "Rules": [
                {
                    "Expiration": {"Days": 1},
                    "Prefix": "prefix",
                    "Status": "Enabled",
                    "ID": "asdfghjklpoiuytrewq",
                }
            ]
        },
        "get_bucket_location": {"LocationConstraint": "EU"},
        "get_bucket_logging": {
            "LoggingEnabled": {"TargetBucket": "my-bucket", "TargetPrefix": "prefix"}
        },
        "get_bucket_notification_configuration": {
            "LambdaFunctionConfigurations": [
                {
                    "LambdaFunctionArn": (
                        "arn:aws:lambda:us-east-1:111111222222:function:my-function"
                    ),
                    "Id": "zxcvbnmlkjhgfdsa",
                    "Events": ["s3:ObjectCreated:*"],
                    "Filter": {
                        "Key": {"FilterRules": [{"Name": "prefix", "Value": "string"}]}
                    },
                }
            ]
        },
        "get_bucket_policy": {
            "Policy": '{"Version":"2012-10-17","Statement":[{"Sid":"","Effect":"Allow","Principal":{"AWS":"arn:aws:iam::111111222222:root"},"Action":"s3:PutObject","Resource":"arn:aws:s3:::my-bucket/*"}]}'
        },
        "get_bucket_replication": {
            "ReplicationConfiguration": {
                "Role": "arn:aws:iam::11111222222:my-role",
                "Rules": [
                    {
                        "ID": "r1",
                        "Prefix": "prefix",
                        "Status": "Enabled",
                        "Destination": {"Bucket": "arn:aws:s3:::my-bucket"},
                    }
                ],
            }
        },
        "get_bucket_request_payment": {"Payer": "Requester"},
        "get_bucket_tagging": {
            "TagSet": [{"Key": "c", "Value": "d"}, {"Key": "a", "Value": "b"}]
        },
        "get_bucket_versioning": {"Status": "Enabled"},
        "get_bucket_website": {
            "ErrorDocument": {"Key": "error.html"},
            "IndexDocument": {"Suffix": "index.html"},
        },
    }


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    "The boto3 module must be greater than or equal to version {}".format(
        required_boto3_version
    ),
)
class BotoS3BucketTestCaseBase(TestCase, LoaderModuleMockMixin):
    conn = None

    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS.copy()
        utils = salt.loader.utils(
            opts, whitelist=["boto3", "args", "systemd", "path", "platform"], context={}
        )
        return {boto_s3_bucket: {"__utils__": utils}}

    def setUp(self):
        super().setUp()
        boto_s3_bucket.__init__(self.opts)
        del self.opts
        # Set up MagicMock to replace the boto3 session
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters["key"] = "".join(
            random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
        )

        self.patcher = patch("boto3.session.Session")
        self.addCleanup(self.patcher.stop)
        self.addCleanup(delattr, self, "patcher")
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        self.addCleanup(delattr, self, "conn")
        session_instance.client.return_value = self.conn


class BotoS3BucketTestCaseMixin:
    pass


class BotoS3BucketTestCase(BotoS3BucketTestCaseBase, BotoS3BucketTestCaseMixin):
    """
    TestCase for salt.modules.boto_s3_bucket module
    """

    def test_that_when_checking_if_a_bucket_exists_and_a_bucket_exists_the_bucket_exists_method_returns_true(
        self,
    ):
        """
        Tests checking s3 bucket existence when the s3 bucket already exists
        """
        self.conn.head_bucket.return_value = None
        result = boto_s3_bucket.exists(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["exists"])

    def test_that_when_checking_if_a_bucket_exists_and_a_bucket_does_not_exist_the_bucket_exists_method_returns_false(
        self,
    ):
        """
        Tests checking s3 bucket existence when the s3 bucket does not exist
        """
        self.conn.head_bucket.side_effect = e404_error
        result = boto_s3_bucket.exists(Bucket="mybucket", **conn_parameters)

        self.assertFalse(result["exists"])

    def test_that_when_checking_if_a_bucket_exists_and_boto3_returns_an_error_the_bucket_exists_method_returns_error(
        self,
    ):
        """
        Tests checking s3 bucket existence when boto returns an error
        """
        self.conn.head_bucket.side_effect = ClientError(error_content, "head_bucket")
        result = boto_s3_bucket.exists(Bucket="mybucket", **conn_parameters)

        self.assertEqual(
            result.get("error", {}).get("message"), error_message.format("head_bucket")
        )

    def test_that_when_creating_a_bucket_succeeds_the_create_bucket_method_returns_true(
        self,
    ):
        """
        tests True bucket created.
        """
        self.conn.create_bucket.return_value = create_ret
        result = boto_s3_bucket.create(
            Bucket="mybucket", LocationConstraint="nowhere", **conn_parameters
        )

        self.assertTrue(result["created"])

    def test_that_when_creating_a_bucket_fails_the_create_bucket_method_returns_error(
        self,
    ):
        """
        tests False bucket not created.
        """
        self.conn.create_bucket.side_effect = ClientError(
            error_content, "create_bucket"
        )
        result = boto_s3_bucket.create(
            Bucket="mybucket", LocationConstraint="nowhere", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("create_bucket"),
        )

    def test_that_when_deleting_a_bucket_succeeds_the_delete_bucket_method_returns_true(
        self,
    ):
        """
        tests True bucket deleted.
        """
        result = boto_s3_bucket.delete(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_a_bucket_fails_the_delete_bucket_method_returns_false(
        self,
    ):
        """
        tests False bucket not deleted.
        """
        self.conn.delete_bucket.side_effect = ClientError(
            error_content, "delete_bucket"
        )
        result = boto_s3_bucket.delete(Bucket="mybucket", **conn_parameters)
        self.assertFalse(result["deleted"])

    def test_that_when_describing_bucket_it_returns_the_dict_of_properties_returns_true(
        self,
    ):
        """
        Tests describing parameters if bucket exists
        """
        for key, value in config_ret.items():
            getattr(self.conn, key).return_value = deepcopy(value)

        result = boto_s3_bucket.describe(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["bucket"])

    def test_that_when_describing_bucket_it_returns_the_dict_of_properties_returns_false(
        self,
    ):
        """
        Tests describing parameters if bucket does not exist
        """
        self.conn.get_bucket_acl.side_effect = not_found_error
        result = boto_s3_bucket.describe(Bucket="mybucket", **conn_parameters)

        self.assertFalse(result["bucket"])

    def test_that_when_describing_bucket_on_client_error_it_returns_error(self):
        """
        Tests describing parameters failure
        """
        self.conn.get_bucket_acl.side_effect = ClientError(
            error_content, "get_bucket_acl"
        )
        result = boto_s3_bucket.describe(Bucket="mybucket", **conn_parameters)
        self.assertTrue("error" in result)

    def test_that_when_listing_buckets_succeeds_the_list_buckets_method_returns_true(
        self,
    ):
        """
        tests True buckets listed.
        """
        self.conn.list_buckets.return_value = deepcopy(list_ret)
        result = boto_s3_bucket.list(**conn_parameters)

        self.assertTrue(result["Buckets"])

    def test_that_when_listing_bucket_fails_the_list_bucket_method_returns_false(self):
        """
        tests False no bucket listed.
        """
        ret = deepcopy(list_ret)
        log.info(ret)
        ret["Buckets"] = list()
        self.conn.list_buckets.return_value = ret
        result = boto_s3_bucket.list(**conn_parameters)
        self.assertFalse(result["Buckets"])

    def test_that_when_listing_bucket_fails_the_list_bucket_method_returns_error(self):
        """
        tests False bucket error.
        """
        self.conn.list_buckets.side_effect = ClientError(error_content, "list_buckets")
        result = boto_s3_bucket.list(**conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"), error_message.format("list_buckets")
        )

    def test_that_when_putting_acl_succeeds_the_put_acl_method_returns_true(self):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_acl(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["updated"])

    def test_that_when_putting_acl_fails_the_put_acl_method_returns_error(self):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_acl.side_effect = ClientError(
            error_content, "put_bucket_acl"
        )
        result = boto_s3_bucket.put_acl(Bucket="mybucket", **conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_acl"),
        )

    def test_that_when_putting_cors_succeeds_the_put_cors_method_returns_true(self):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_cors(
            Bucket="mybucket", CORSRules="[]", **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_putting_cors_fails_the_put_cors_method_returns_error(self):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_cors.side_effect = ClientError(
            error_content, "put_bucket_cors"
        )
        result = boto_s3_bucket.put_cors(
            Bucket="mybucket", CORSRules="[]", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_cors"),
        )

    def test_that_when_putting_lifecycle_configuration_succeeds_the_put_lifecycle_configuration_method_returns_true(
        self,
    ):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_lifecycle_configuration(
            Bucket="mybucket", Rules="[]", **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_putting_lifecycle_configuration_fails_the_put_lifecycle_configuration_method_returns_error(
        self,
    ):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_lifecycle_configuration.side_effect = ClientError(
            error_content, "put_bucket_lifecycle_configuration"
        )
        result = boto_s3_bucket.put_lifecycle_configuration(
            Bucket="mybucket", Rules="[]", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_lifecycle_configuration"),
        )

    def test_that_when_putting_logging_succeeds_the_put_logging_method_returns_true(
        self,
    ):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_logging(
            Bucket="mybucket",
            TargetBucket="arn:::::",
            TargetPrefix="asdf",
            TargetGrants="[]",
            **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_putting_logging_fails_the_put_logging_method_returns_error(self):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_logging.side_effect = ClientError(
            error_content, "put_bucket_logging"
        )
        result = boto_s3_bucket.put_logging(
            Bucket="mybucket",
            TargetBucket="arn:::::",
            TargetPrefix="asdf",
            TargetGrants="[]",
            **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_logging"),
        )

    def test_that_when_putting_notification_configuration_succeeds_the_put_notification_configuration_method_returns_true(
        self,
    ):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_notification_configuration(
            Bucket="mybucket", **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_putting_notification_configuration_fails_the_put_notification_configuration_method_returns_error(
        self,
    ):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_notification_configuration.side_effect = ClientError(
            error_content, "put_bucket_notification_configuration"
        )
        result = boto_s3_bucket.put_notification_configuration(
            Bucket="mybucket", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_notification_configuration"),
        )

    def test_that_when_putting_policy_succeeds_the_put_policy_method_returns_true(self):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_policy(
            Bucket="mybucket", Policy="{}", **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_putting_policy_fails_the_put_policy_method_returns_error(self):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_policy.side_effect = ClientError(
            error_content, "put_bucket_policy"
        )
        result = boto_s3_bucket.put_policy(
            Bucket="mybucket", Policy="{}", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_policy"),
        )

    def test_that_when_putting_replication_succeeds_the_put_replication_method_returns_true(
        self,
    ):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_replication(
            Bucket="mybucket", Role="arn:aws:iam:::", Rules="[]", **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_putting_replication_fails_the_put_replication_method_returns_error(
        self,
    ):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_replication.side_effect = ClientError(
            error_content, "put_bucket_replication"
        )
        result = boto_s3_bucket.put_replication(
            Bucket="mybucket", Role="arn:aws:iam:::", Rules="[]", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_replication"),
        )

    def test_that_when_putting_request_payment_succeeds_the_put_request_payment_method_returns_true(
        self,
    ):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_request_payment(
            Bucket="mybucket", Payer="Requester", **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_putting_request_payment_fails_the_put_request_payment_method_returns_error(
        self,
    ):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_request_payment.side_effect = ClientError(
            error_content, "put_bucket_request_payment"
        )
        result = boto_s3_bucket.put_request_payment(
            Bucket="mybucket", Payer="Requester", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_request_payment"),
        )

    def test_that_when_putting_tagging_succeeds_the_put_tagging_method_returns_true(
        self,
    ):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_tagging(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["updated"])

    def test_that_when_putting_tagging_fails_the_put_tagging_method_returns_error(self):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_tagging.side_effect = ClientError(
            error_content, "put_bucket_tagging"
        )
        result = boto_s3_bucket.put_tagging(Bucket="mybucket", **conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_tagging"),
        )

    def test_that_when_putting_versioning_succeeds_the_put_versioning_method_returns_true(
        self,
    ):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_versioning(
            Bucket="mybucket", Status="Enabled", **conn_parameters
        )

        self.assertTrue(result["updated"])

    def test_that_when_putting_versioning_fails_the_put_versioning_method_returns_error(
        self,
    ):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_versioning.side_effect = ClientError(
            error_content, "put_bucket_versioning"
        )
        result = boto_s3_bucket.put_versioning(
            Bucket="mybucket", Status="Enabled", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_versioning"),
        )

    def test_that_when_putting_website_succeeds_the_put_website_method_returns_true(
        self,
    ):
        """
        tests True bucket updated.
        """
        result = boto_s3_bucket.put_website(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["updated"])

    def test_that_when_putting_website_fails_the_put_website_method_returns_error(self):
        """
        tests False bucket not updated.
        """
        self.conn.put_bucket_website.side_effect = ClientError(
            error_content, "put_bucket_website"
        )
        result = boto_s3_bucket.put_website(Bucket="mybucket", **conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("put_bucket_website"),
        )

    def test_that_when_deleting_cors_succeeds_the_delete_cors_method_returns_true(self):
        """
        tests True bucket attribute deleted.
        """
        result = boto_s3_bucket.delete_cors(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_cors_fails_the_delete_cors_method_returns_error(self):
        """
        tests False bucket attribute not deleted.
        """
        self.conn.delete_bucket_cors.side_effect = ClientError(
            error_content, "delete_bucket_cors"
        )
        result = boto_s3_bucket.delete_cors(Bucket="mybucket", **conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("delete_bucket_cors"),
        )

    def test_that_when_deleting_lifecycle_configuration_succeeds_the_delete_lifecycle_configuration_method_returns_true(
        self,
    ):
        """
        tests True bucket attribute deleted.
        """
        result = boto_s3_bucket.delete_lifecycle_configuration(
            Bucket="mybucket", **conn_parameters
        )

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_lifecycle_configuration_fails_the_delete_lifecycle_configuration_method_returns_error(
        self,
    ):
        """
        tests False bucket attribute not deleted.
        """
        self.conn.delete_bucket_lifecycle.side_effect = ClientError(
            error_content, "delete_bucket_lifecycle_configuration"
        )
        result = boto_s3_bucket.delete_lifecycle_configuration(
            Bucket="mybucket", **conn_parameters
        )
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("delete_bucket_lifecycle_configuration"),
        )

    def test_that_when_deleting_policy_succeeds_the_delete_policy_method_returns_true(
        self,
    ):
        """
        tests True bucket attribute deleted.
        """
        result = boto_s3_bucket.delete_policy(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_policy_fails_the_delete_policy_method_returns_error(
        self,
    ):
        """
        tests False bucket attribute not deleted.
        """
        self.conn.delete_bucket_policy.side_effect = ClientError(
            error_content, "delete_bucket_policy"
        )
        result = boto_s3_bucket.delete_policy(Bucket="mybucket", **conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("delete_bucket_policy"),
        )

    def test_that_when_deleting_replication_succeeds_the_delete_replication_method_returns_true(
        self,
    ):
        """
        tests True bucket attribute deleted.
        """
        result = boto_s3_bucket.delete_replication(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_replication_fails_the_delete_replication_method_returns_error(
        self,
    ):
        """
        tests False bucket attribute not deleted.
        """
        self.conn.delete_bucket_replication.side_effect = ClientError(
            error_content, "delete_bucket_replication"
        )
        result = boto_s3_bucket.delete_replication(Bucket="mybucket", **conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("delete_bucket_replication"),
        )

    def test_that_when_deleting_tagging_succeeds_the_delete_tagging_method_returns_true(
        self,
    ):
        """
        tests True bucket attribute deleted.
        """
        result = boto_s3_bucket.delete_tagging(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_tagging_fails_the_delete_tagging_method_returns_error(
        self,
    ):
        """
        tests False bucket attribute not deleted.
        """
        self.conn.delete_bucket_tagging.side_effect = ClientError(
            error_content, "delete_bucket_tagging"
        )
        result = boto_s3_bucket.delete_tagging(Bucket="mybucket", **conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("delete_bucket_tagging"),
        )

    def test_that_when_deleting_website_succeeds_the_delete_website_method_returns_true(
        self,
    ):
        """
        tests True bucket attribute deleted.
        """
        result = boto_s3_bucket.delete_website(Bucket="mybucket", **conn_parameters)

        self.assertTrue(result["deleted"])

    def test_that_when_deleting_website_fails_the_delete_website_method_returns_error(
        self,
    ):
        """
        tests False bucket attribute not deleted.
        """
        self.conn.delete_bucket_website.side_effect = ClientError(
            error_content, "delete_bucket_website"
        )
        result = boto_s3_bucket.delete_website(Bucket="mybucket", **conn_parameters)
        self.assertEqual(
            result.get("error", {}).get("message"),
            error_message.format("delete_bucket_website"),
        )



"""
Netmiko
=======

.. versionadded:: 2019.2.0

Proxy module for managing network devices via
`Netmiko <https://github.com/ktbyers/netmiko>`_.

:codeauthor: Mircea Ulinic <ping@mirceaulinic.net> & Kirk Byers <ktbyers@twb-tech.com>
:maturity:   new
:depends:    netmiko
:platform:   unix

Dependencies
------------

The ``netmiko`` proxy modules requires Netmiko to be installed: ``pip install netmiko``.

Pillar
------

The ``netmiko`` proxy configuration requires the following parameters in order
to connect to the network device:

- ``device_type`` - Class selection based on device type. Supported options:

  - ``a10``: A10 Networks
  - ``accedian``: Accedian Networks
  - ``alcatel_aos``: Alcatel AOS
  - ``alcatel_sros``: Alcatel SROS
  - ``apresia_aeos``: Apresia AEOS
  - ``arista_eos``: Arista EOS
  - ``aruba_os``: Aruba
  - ``avaya_ers``: Avaya ERS
  - ``avaya_vsp``: Avaya VSP
  - ``brocade_fastiron``: Brocade Fastiron
  - ``brocade_netiron``: Brocade Netiron
  - ``brocade_nos``: Brocade NOS
  - ``brocade_vdx``: Brocade NOS
  - ``brocade_vyos``: VyOS
  - ``checkpoint_gaia``: Check Point GAiA
  - ``calix_b6``: Calix B6
  - ``ciena_saos``: Ciena SAOS
  - ``cisco_asa``: Cisco SA
  - ``cisco_ios``: Cisco IOS
  - ``cisco_nxos``: Cisco NX-oS
  - ``cisco_s300``: Cisco S300
  - ``cisco_tp``: Cisco TpTcCe
  - ``cisco_wlc``: Cisco WLC
  - ``cisco_xe``: Cisco IOS
  - ``cisco_xr``: Cisco XR
  - ``coriant``: Coriant
  - ``dell_force10``: Dell Force10
  - ``dell_os10``: Dell OS10
  - ``dell_powerconnect``: Dell PowerConnect
  - ``eltex``: Eltex
  - ``enterasys``: Enterasys
  - ``extreme``: Extreme
  - ``extreme_wing``: Extreme Wing
  - ``f5_ltm``: F5 LTM
  - ``fortinet``: Fortinet
  - ``generic_termserver``: TerminalServer
  - ``hp_comware``: HP Comware
  - ``hp_procurve``: HP Procurve
  - ``huawei``: Huawei
  - ``huawei_vrpv8``: Huawei VRPV8
  - ``juniper``: Juniper Junos
  - ``juniper_junos``: Juniper Junos
  - ``linux``: Linux
  - ``mellanox``: Mellanox
  - ``mrv_optiswitch``: MrvOptiswitch
  - ``netapp_cdot``: NetAppcDot
  - ``netscaler``: Netscaler
  - ``ovs_linux``: OvsLinux
  - ``paloalto_panos``: PaloAlto Panos
  - ``pluribus``: Pluribus
  - ``quanta_mesh``: Quanta Mesh
  - ``ruckus_fastiron``: Ruckus Fastiron
  - ``ubiquiti_edge``: Ubiquiti Edge
  - ``ubiquiti_edgeswitch``: Ubiquiti Edge
  - ``vyatta_vyos``: VyOS
  - ``vyos``: VyOS
  - ``brocade_fastiron_telnet``: Brocade Fastiron over Telnet
  - ``brocade_netiron_telnet``: Brocade Netiron over Telnet
  - ``cisco_ios_telnet``: Cisco IOS over Telnet
  - ``apresia_aeos_telnet``: Apresia AEOS over Telnet
  - ``arista_eos_telnet``: Arista EOS over Telnet
  - ``hp_procurve_telnet``: HP Procurve over Telnet
  - ``hp_comware_telnet``: HP Comware over Telnet
  - ``juniper_junos_telnet``: Juniper Junos over Telnet
  - ``calix_b6_telnet``: Calix B6 over Telnet
  - ``dell_powerconnect_telnet``: Dell PowerConnect over Telnet
  - ``generic_termserver_telnet``: TerminalServer over Telnet
  - ``extreme_telnet``: Extreme Networks over Telnet
  - ``ruckus_fastiron_telnet``: Ruckus Fastiron over Telnet
  - ``cisco_ios_serial``: Cisco IOS over serial port

- ``ip`` - IP address of target device (not required if ``host`` is provided)

- ``host`` - Hostname of target device (not required if ``ip`` is provided)

- ``username`` - Username to authenticate against target device, if required

- ``password`` - Password to authenticate against target device, if required

- ``secret`` - The enable password if target device requires one

- ``port`` - The destination port used to connect to the target device

- ``global_delay_factor`` - Multiplication factor affecting Netmiko delays
  (default: ``1``)

- ``use_keys`` - Connect to target device using SSH keys (default: ``False``)

- ``key_file`` - Filename path of the SSH key file to use

- ``allow_agent`` - Enable use of SSH key-agent

- ``ssh_strict`` - Automatically reject unknown SSH host keys (default:
  ``False``, which means unknown SSH host keys will be accepted)

- ``system_host_keys`` - Load host keys from the user's "known_hosts" file
  (default: ``False``)

- ``alt_host_keys`` - If ``True``,  host keys will be loaded from the file
  specified in ``alt_key_file`` (default: ``False``)

- ``alt_key_file`` - SSH host key file to use (if ``alt_host_keys=True``)

- ``ssh_config_file`` - File name of OpenSSH configuration file

- ``timeout`` - Connection timeout, in seconds (default: ``90``)

- ``session_timeout`` - Set a timeout for parallel requests, in seconds
  (default: ``60``)

- ``keepalive`` - Send SSH keepalive packets at a specific interval, in
  seconds. Currently defaults to ``0``, for backwards compatibility (it will
  not attempt to keep the connection alive using the KEEPALIVE packets).

- ``default_enter`` - Character(s) to send to correspond to enter key (default:
  ``\\n``)

- ``response_return`` - Character(s) to use in normalized return data to
  represent enter key (default: ``\\n``)

- ``always_alive`` - In certain less dynamic environments, maintaining the
  remote connection permanently open with the network device is not always
  beneficial. In that case, the user can select to initialize the connection
  only when needed, by setting this option to ``False``. By default this option
  is set to ``True`` (maintains the connection with the remote network device)

- ``multiprocessing`` - Overrides the :conf_minion:`multiprocessing` option,
  per proxy minion, as the Netmiko communication channel is mainly SSH
  (default: ``False``)

- ``connection_timeout`` - The number of seconds to attempt to connect to
  the device in seconds.
  (default: ``300``)

Proxy Pillar Example
--------------------

.. code-block:: yaml

    proxy:
      proxytype: netmiko
      device_type: juniper_junos
      host: router1.example.com
      username: example
      password: example

.. code-block:: yaml

    proxy:
      proxytype: netmiko
      device_type: cisco_ios
      ip: 1.2.3.4
      username: test
      use_keys: true
      secret: w3@k
"""

import contextlib
import logging
import time

import salt.utils.args

try:
    from netmiko import ConnectHandler
    from netmiko.ssh_exception import NetMikoTimeoutException
    from netmiko.ssh_exception import NetMikoAuthenticationException

    HAS_NETMIKO = True
except ImportError:
    HAS_NETMIKO = False


# -----------------------------------------------------------------------------
# proxy properties
# -----------------------------------------------------------------------------

__proxyenabled__ = ["netmiko"]
# proxy name

# -----------------------------------------------------------------------------
# globals
# -----------------------------------------------------------------------------

__virtualname__ = "netmiko"
log = logging.getLogger(__name__)
netmiko_device = {}

DEFAULT_CONNECTION_TIMEOUT = 300

# -----------------------------------------------------------------------------
# propery functions
# -----------------------------------------------------------------------------


def __virtual__():
    """
    Proxy module available only if Netmiko is installed.
    """
    if not HAS_NETMIKO:
        return (
            False,
            "The netmiko proxy module requires netmiko library to be installed.",
        )
    return __virtualname__


# -----------------------------------------------------------------------------
# proxy functions
# -----------------------------------------------------------------------------


def init(opts):
    """
    Open the connection to the network device
    managed through netmiko.
    """
    __context__["netmiko_device"] = {}
    __context__["netmiko_device"]["opts"] = opts
    __context__["netmiko_device"]["id"] = opts["id"]
    log.debug("Init for %s", opts["id"])
    proxy_dict = opts.get("proxy", {})
    skip_connect = opts.get(
        "skip_connect_on_init", proxy_dict.get("skip_connect_on_init", False)
    )
    opts["multiprocessing"] = proxy_dict.get(
        "multiprocessing", opts.get("multiprocessing", False)
    )
    __context__["netmiko_device"]["connection_timeout"] = opts.get(
        "connection_timeout", DEFAULT_CONNECTION_TIMEOUT
    )

    netmiko_connection_args = proxy_dict.copy()
    netmiko_connection_args.pop("proxytype", None)
    netmiko_connection_args.pop("multiprocessing", None)
    netmiko_connection_args.pop("skip_connect_on_init", None)

    __context__["netmiko_device"]["args"] = netmiko_connection_args

    _always_alive = netmiko_connection_args.pop(
        "always_alive", opts.get("proxy_always_alive", True)
    )
    __context__["netmiko_device"]["always_alive"] = _always_alive

    if not skip_connect:
        try:
            with make_con() as con:
                __context__["netmiko_device"]["connection"] = con
                __context__["netmiko_device"]["initialized"] = True
                __context__["netmiko_device"]["up"] = True
        except NetMikoTimeoutException as t_err:
            log.error("Unable to setup the netmiko connection", exc_info=True)
        except NetMikoAuthenticationException as au_err:
            log.error("Unable to setup the netmiko connection", exc_info=True)
    else:
        __context__["netmiko_device"]["up"] = True
        __context__["netmiko_device"]["initialized"] = False
        return True


def make_con(connection_timeout=DEFAULT_CONNECTION_TIMEOUT):
    log.error("Creating connection to %s", __context__["netmiko_device"]["id"])
    args = __context__["netmiko_device"]["args"]
    start = time.time()
    args = args.copy()
    found_exception = None
    connection = None
    while True:
        try:
            connection = ConnectHandler(**args)
        except Exception as exc:  # pylint: disable=broad-except
            log.warning("Got exception %r", exc)
            found_exception = exc
        else:
            break
        if time.time() - start >= connection_timeout:
            if found_exception:
                raise found_exception
            else:
                raise Exception("Unable to create connection")
    return connection


@contextlib.contextmanager
def connection(connection_timeout=DEFAULT_CONNECTION_TIMEOUT):
    if "connection" in __context__["netmiko_device"]:
        con = __context__["netmiko_device"]["connection"]
        if con.remote_conn is None:
            con = make_con(connection_timeout)
            __context__["netmiko_device"]["connection"] = con
        if con.remote_conn.closed:
            con = make_con()
            __context__["netmiko_device"]["connection"] = con
    else:
        con = make_con(connection_timeout)
        __context__["netmiko_device"]["connection"] = con
    __context__["netmiko_device"]["initialized"] = True
    try:
        yield con
    finally:
        if not __context__["netmiko_device"]["always_alive"]:
            con.disconnect()


def alive(opts):
    """
    Return the connection status with the network device.
    """
    log.debug("Checking if %s is still alive", opts.get("id", ""))
    connection_timeout = __context__["netmiko_device"]["connection_timeout"]

    if not __context__["netmiko_device"]["always_alive"]:
        return True
    if ping() and initialized():
        with connection(connection_timeout) as con:
            return con.remote_conn.transport.is_alive()
    return False


def ping():
    """
    Connection open successfully?
    """
    return __context__["netmiko_device"].get("up", False)


def initialized():
    """
    Connection finished initializing?
    """
    return __context__["netmiko_device"].get("initialized", False)


def shutdown(opts):
    """
    Closes connection with the device.
    """
    return call("disconnect")


# -----------------------------------------------------------------------------
# callable functions
# -----------------------------------------------------------------------------


def conn():
    """
    Return the connection object.
    """
    return __context__["netmiko_device"].get("connection")


def args():
    """
    Return the Netmiko device args.
    """
    return __context__["netmiko_device"]["args"]


def call(method, *args, **kwargs):
    """
    Calls an arbitrary netmiko method.
    """
    kwargs = salt.utils.args.clean_kwargs(**kwargs)
    connection_timeout = __context__["netmiko_device"]["connection_timeout"]

    with connection(connection_timeout) as con:
        return getattr(con, method)(*args, **kwargs)



"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

import pytest
import salt.modules.win_service as win_service
import salt.utils.path
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

try:
    WINAPI = True
    import win32serviceutil
    import pywintypes
except ImportError:
    WINAPI = False


class WinServiceTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.win_service
    """

    def setup_loader_modules(self):
        return {win_service: {}}

    def test_get_enabled(self):
        """
        Test to return the enabled services
        """
        mock = MagicMock(
            return_value=[
                {"ServiceName": "spongebob"},
                {"ServiceName": "squarepants"},
                {"ServiceName": "patrick"},
            ]
        )
        with patch.object(win_service, "_get_services", mock):
            mock_info = MagicMock(
                side_effect=[
                    {"StartType": "Auto"},
                    {"StartType": "Manual"},
                    {"StartType": "Disabled"},
                ]
            )
            with patch.object(win_service, "info", mock_info):
                self.assertListEqual(win_service.get_enabled(), ["spongebob"])

    def test_get_disabled(self):
        """
        Test to return the disabled services
        """
        mock = MagicMock(
            return_value=[
                {"ServiceName": "spongebob"},
                {"ServiceName": "squarepants"},
                {"ServiceName": "patrick"},
            ]
        )
        with patch.object(win_service, "_get_services", mock):
            mock_info = MagicMock(
                side_effect=[
                    {"StartType": "Auto"},
                    {"StartType": "Manual"},
                    {"StartType": "Disabled"},
                ]
            )
            with patch.object(win_service, "info", mock_info):
                self.assertListEqual(
                    win_service.get_disabled(), ["patrick", "squarepants"]
                )

    def test_available(self):
        """
        Test to Returns ``True`` if the specified service
        is available, otherwise returns ``False``
        """
        mock = MagicMock(return_value=["c", "a", "b"])
        with patch.object(win_service, "get_all", mock):
            self.assertTrue(win_service.available("a"))

    def test_missing(self):
        """
        Test to the inverse of service.available
        """
        mock = MagicMock(return_value=["c", "a", "b"])
        with patch.object(win_service, "get_all", mock):
            self.assertTrue(win_service.missing("d"))

    def test_get_all(self):
        """
        Test to return all installed services
        """
        mock = MagicMock(
            return_value=[
                {"ServiceName": "spongebob"},
                {"ServiceName": "squarepants"},
                {"ServiceName": "patrick"},
            ]
        )
        with patch.object(win_service, "_get_services", mock):
            self.assertListEqual(
                win_service.get_all(), ["patrick", "spongebob", "squarepants"]
            )

    def test_get_service_name(self):
        """
        Test to the Display Name is what is displayed
        in Windows when services.msc is executed.
        """
        mock = MagicMock(
            return_value=[
                {"ServiceName": "spongebob", "DisplayName": "Sponge Bob"},
                {"ServiceName": "squarepants", "DisplayName": "Square Pants"},
                {"ServiceName": "patrick", "DisplayName": "Patrick the Starfish"},
            ]
        )
        with patch.object(win_service, "_get_services", mock):
            self.assertDictEqual(
                win_service.get_service_name(),
                {
                    "Patrick the Starfish": "patrick",
                    "Sponge Bob": "spongebob",
                    "Square Pants": "squarepants",
                },
            )
            self.assertDictEqual(
                win_service.get_service_name("patrick"),
                {"Patrick the Starfish": "patrick"},
            )

    @skipIf(not WINAPI, "win32serviceutil not available")
    @pytest.mark.slow_test
    def test_start(self):
        """
        Test to start the specified service
        """
        mock_true = MagicMock(return_value=True)
        mock_false = MagicMock(return_value=False)
        mock_info = MagicMock(side_effect=[{"Status": "Running"}])

        with patch.object(win32serviceutil, "StartService", mock_true), patch.object(
            win_service, "disabled", mock_false
        ), patch.object(win_service, "info", mock_info):
            self.assertTrue(win_service.start("spongebob"))

        mock_info = MagicMock(
            side_effect=[
                {"Status": "Stopped", "Status_WaitHint": 0},
                {"Status": "Start Pending", "Status_WaitHint": 0},
                {"Status": "Running"},
            ]
        )

        with patch.object(win32serviceutil, "StartService", mock_true), patch.object(
            win_service, "disabled", mock_false
        ), patch.object(win_service, "info", mock_info), patch.object(
            win_service, "status", mock_true
        ):
            self.assertTrue(win_service.start("spongebob"))

    @skipIf(not WINAPI, "pywintypes not available")
    def test_start_already_running(self):
        """
        Test starting a service that is already running
        """
        mock_false = MagicMock(return_value=False)
        mock_error = MagicMock(
            side_effect=pywintypes.error(1056, "StartService", "Service is running")
        )
        mock_info = MagicMock(side_effect=[{"Status": "Running"}])
        with patch.object(win32serviceutil, "StartService", mock_error), patch.object(
            win_service, "disabled", mock_false
        ), patch.object(win_service, "_status_wait", mock_info):
            self.assertTrue(win_service.start("spongebob"))

    @skipIf(not WINAPI, "win32serviceutil not available")
    @pytest.mark.slow_test
    def test_stop(self):
        """
        Test to stop the specified service
        """
        mock_true = MagicMock(return_value=True)
        mock_false = MagicMock(return_value=False)
        mock_info = MagicMock(side_effect=[{"Status": "Stopped"}])

        with patch.object(win32serviceutil, "StopService", mock_true), patch.object(
            win_service, "_status_wait", mock_info
        ):
            self.assertTrue(win_service.stop("spongebob"))

        mock_info = MagicMock(
            side_effect=[
                {"Status": "Running", "Status_WaitHint": 0},
                {"Status": "Stop Pending", "Status_WaitHint": 0},
                {"Status": "Stopped"},
            ]
        )

        with patch.object(win32serviceutil, "StopService", mock_true), patch.object(
            win_service, "info", mock_info
        ), patch.object(win_service, "status", mock_false):
            self.assertTrue(win_service.stop("spongebob"))

    @skipIf(not WINAPI, "pywintypes not available")
    def test_stop_not_running(self):
        """
        Test stopping a service that is already stopped
        """
        mock_error = MagicMock(
            side_effect=pywintypes.error(1062, "StopService", "Service is not running")
        )
        mock_info = MagicMock(side_effect=[{"Status": "Stopped"}])
        with patch.object(win32serviceutil, "StopService", mock_error), patch.object(
            win_service, "_status_wait", mock_info
        ):
            self.assertTrue(win_service.stop("spongebob"))

    def test_restart(self):
        """
        Test to restart the named service
        """
        mock_true = MagicMock(return_value=True)
        with patch.object(win_service, "create_win_salt_restart_task", mock_true):
            with patch.object(win_service, "execute_salt_restart_task", mock_true):
                self.assertTrue(win_service.restart("salt-minion"))

        with patch.object(win_service, "stop", mock_true):
            with patch.object(win_service, "start", mock_true):
                self.assertTrue(win_service.restart("salt"))

    def test_createwin_saltrestart_task(self):
        """
        Test to create a task in Windows task
        scheduler to enable restarting the salt-minion
        """
        cmd = salt.utils.path.which("cmd")
        mock = MagicMock()
        with patch.dict(win_service.__salt__, {"task.create_task": mock}):
            win_service.create_win_salt_restart_task()
            mock.assert_called_once_with(
                action_type="Execute",
                arguments=(
                    "/c ping -n 3 127.0.0.1 && net stop salt-minion && "
                    "net start salt-minion"
                ),
                cmd=cmd,
                force=True,
                name="restart-salt-minion",
                start_date="1975-01-01",
                start_time="01:00",
                trigger_type="Once",
                user_name="System",
            )

    def test_execute_salt_restart_task(self):
        """
        Test to run the Windows Salt restart task
        """
        mock_true = MagicMock(return_value=True)
        with patch.dict(win_service.__salt__, {"task.run": mock_true}):
            self.assertTrue(win_service.execute_salt_restart_task())

    @skipIf(not WINAPI, "win32serviceutil not available")
    def test_status(self):
        """
        Test to return the status for a service
        """
        mock_info = MagicMock(
            side_effect=[
                {"Status": "Running"},
                {"Status": "Stop Pending"},
                {"Status": "Stopped"},
            ]
        )

        with patch.object(win_service, "info", mock_info):
            self.assertTrue(win_service.status("spongebob"))
            self.assertTrue(win_service.status("patrick"))
            self.assertFalse(win_service.status("squidward"))

    def test_getsid(self):
        """
        Test to return the sid for this windows service
        """
        mock_info = MagicMock(
            side_effect=[{"sid": "S-1-5-80-1956725871..."}, {"sid": None}]
        )
        with patch.object(win_service, "info", mock_info):
            self.assertEqual(win_service.getsid("spongebob"), "S-1-5-80-1956725871...")
            self.assertEqual(win_service.getsid("plankton"), None)

    def test_enable(self):
        """
        Test to enable the named service to start at boot
        """
        mock_modify = MagicMock(return_value=True)
        mock_info = MagicMock(
            return_value={"StartType": "Auto", "StartTypeDelayed": False}
        )
        with patch.object(win_service, "modify", mock_modify):
            with patch.object(win_service, "info", mock_info):
                self.assertTrue(win_service.enable("spongebob"))

    def test_disable(self):
        """
        Test to disable the named service to start at boot
        """
        mock_modify = MagicMock(return_value=True)
        mock_info = MagicMock(return_value={"StartType": "Disabled"})
        with patch.object(win_service, "modify", mock_modify):
            with patch.object(win_service, "info", mock_info):
                self.assertTrue(win_service.disable("spongebob"))

    def test_enabled(self):
        """
        Test to check to see if the named
        service is enabled to start on boot
        """
        mock = MagicMock(side_effect=[{"StartType": "Auto"}, {"StartType": "Disabled"}])
        with patch.object(win_service, "info", mock):
            self.assertTrue(win_service.enabled("spongebob"))
            self.assertFalse(win_service.enabled("squarepants"))

    def test_enabled_with_space_in_name(self):
        """
        Test to check to see if the named
        service is enabled to start on boot
        when have space in service name
        """
        mock = MagicMock(side_effect=[{"StartType": "Auto"}, {"StartType": "Disabled"}])
        with patch.object(win_service, "info", mock):
            self.assertTrue(win_service.enabled("spongebob test"))
            self.assertFalse(win_service.enabled("squarepants test"))

    def test_disabled(self):
        """
        Test to check to see if the named
        service is disabled to start on boot
        """
        mock = MagicMock(side_effect=[False, True])
        with patch.object(win_service, "enabled", mock):
            self.assertTrue(win_service.disabled("spongebob"))
            self.assertFalse(win_service.disabled("squarepants"))

    def test_cmd_quote(self):
        """
        Make sure the command gets quoted correctly
        """
        # Should always return command wrapped in double quotes
        expected = r'"C:\Program Files\salt\test.exe"'

        # test no quotes
        bin_path = r"C:\Program Files\salt\test.exe"
        self.assertEqual(win_service._cmd_quote(bin_path), expected)

        # test single quotes
        bin_path = r"'C:\Program Files\salt\test.exe'"
        self.assertEqual(win_service._cmd_quote(bin_path), expected)

        # test double quoted single quotes
        bin_path = "\"'C:\\Program Files\\salt\\test.exe'\""
        self.assertEqual(win_service._cmd_quote(bin_path), expected)

        # test single quoted, double quoted, single quotes
        bin_path = "'\"'C:\\Program Files\\salt\\test.exe'\"'"
        self.assertEqual(win_service._cmd_quote(bin_path), expected)



from tests.support.mock import call


def network_update_call(
    name,
    bridge,
    forward,
    vport=None,
    tag=None,
    ipv4_config=None,
    ipv6_config=None,
    connection=None,
    username=None,
    password=None,
    mtu=None,
    domain=None,
    nat=None,
    interfaces=None,
    addresses=None,
    physical_function=None,
    dns=None,
    test=False,
):
    """
    Create a call object with the missing default parameters from virt.network_update()
    """
    return call(
        name,
        bridge,
        forward,
        vport=vport,
        tag=tag,
        ipv4_config=ipv4_config,
        ipv6_config=ipv6_config,
        mtu=mtu,
        domain=domain,
        nat=nat,
        interfaces=interfaces,
        addresses=addresses,
        physical_function=physical_function,
        dns=dns,
        test=test,
        connection=connection,
        username=username,
        password=password,
    )


def domain_update_call(
    name,
    cpu=None,
    mem=None,
    disk_profile=None,
    disks=None,
    nic_profile=None,
    interfaces=None,
    graphics=None,
    connection=None,
    username=None,
    password=None,
    boot=None,
    numatune=None,
    boot_dev=None,
    hypervisor_features=None,
    clock=None,
    serials=None,
    consoles=None,
    stop_on_reboot=False,
    live=True,
    host_devices=None,
    test=False,
):
    """
    Create a call object with the missing default parameters from virt.update()
    """
    return call(
        name,
        cpu=cpu,
        mem=mem,
        disk_profile=disk_profile,
        disks=disks,
        nic_profile=nic_profile,
        interfaces=interfaces,
        graphics=graphics,
        live=live,
        connection=connection,
        username=username,
        password=password,
        boot=boot,
        numatune=numatune,
        serials=serials,
        consoles=consoles,
        test=test,
        boot_dev=boot_dev,
        hypervisor_features=hypervisor_features,
        clock=clock,
        stop_on_reboot=stop_on_reboot,
        host_devices=host_devices,
    )



#!/usr/bin/env python3
# see issue: https://gitlab.com/saltstack/open/salt-pkg/-/issues/19
import _pyio
import contextlib
import multiprocessing
import os
import pathlib
import sys

import salt.scripts
import salt.utils.platform

# tiamat pip breaks singlebin on Windows at the moment
# https://gitlab.com/saltstack/pop/tiamat-pip/-/issues/4
if not sys.platform.startswith("win"):
    import tiamatpip.cli
    import tiamatpip.configure
    import tiamatpip.utils

AVAIL = (
    "minion",
    "master",
    "call",
    "api",
    "cloud",
    "cp",
    "extend",
    "key",
    "proxy",
    "pip",
    "run",
    "shell",
    "spm",
    "ssh",
    "support",
    "syndic",
)


# tiamat pip breaks singlebin on Windows at the moment
# https://gitlab.com/saltstack/pop/tiamat-pip/-/issues/4
if not sys.platform.startswith("win"):
    PIP_PATH = pathlib.Path(f"{os.sep}opt", "saltstack", "salt", "pypath")
    with contextlib.suppress(PermissionError):
        PIP_PATH.mkdir(mode=0o755, parents=True, exist_ok=True)
    tiamatpip.configure.set_user_site_packages_path(PIP_PATH)


def redirect(argv):
    """
    Change the args and redirect to another salt script
    """
    if len(argv) < 2:
        msg = "Must pass in a salt command, available commands are:"
        for cmd in AVAIL:
            msg += f"\n{cmd}"
        print(msg, file=sys.stderr, flush=True)
        sys.exit(1)
    cmd = sys.argv[1]
    if cmd == "shell":
        py_shell()
        return
    # tiamat pip breaks singlebin on Windows at the moment
    # https://gitlab.com/saltstack/pop/tiamat-pip/-/issues/4
    if not sys.platform.startswith("win"):
        if tiamatpip.cli.should_redirect_argv(argv):
            tiamatpip.cli.process_pip_argv(argv)
            return
    if cmd not in AVAIL:
        # Fall back to the salt command
        args = ["salt"]
        s_fun = salt.scripts.salt_main
    else:
        args = [f"salt-{cmd}"]
        sys.argv.pop(1)
        s_fun = getattr(salt.scripts, f"salt_{cmd}")
    args.extend(argv[1:])
    # tiamat pip breaks singlebin on Windows at the moment
    # https://gitlab.com/saltstack/pop/tiamat-pip/-/issues/4
    if not sys.platform.startswith("win"):
        with tiamatpip.utils.patched_sys_argv(args):
            s_fun()
    else:
        s_fun()


def py_shell():
    if not sys.platform.startswith("win"):
        import readline  # optional, will allow Up/Down/History in the console
    import code

    variables = globals().copy()
    variables.update(locals())
    shell = code.InteractiveConsole(variables)
    shell.interact()


if __name__ == "__main__":
    if sys.platform.startswith("win"):
        multiprocessing.freeze_support()
    redirect(sys.argv)



"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""

import os
import textwrap

import salt.modules.mount as mount
import salt.utils.files
import salt.utils.path
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase

MOCK_SHELL_FILE = "A B C D F G\n"


class MountTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.mount
    """

    def setup_loader_modules(self):
        return {mount: {}}

    def test_active(self):
        """
        List the active mounts.
        """
        with patch.dict(mount.__grains__, {"os": "FreeBSD", "kernel": "FreeBSD"}):
            # uid=user1 tests the improbable case where a OS returns a name
            # instead of a numeric id, for #25293
            mock = MagicMock(return_value="A B C D,E,F,uid=user1,gid=grp1")
            mock_user = MagicMock(return_value={"uid": "100"})
            mock_group = MagicMock(return_value={"gid": "100"})
            with patch.dict(
                mount.__salt__,
                {
                    "cmd.run_stdout": mock,
                    "user.info": mock_user,
                    "group.info": mock_group,
                },
            ):
                self.assertEqual(
                    mount.active(),
                    {
                        "B": {
                            "device": "A",
                            "opts": ["D", "E", "F", "uid=100", "gid=100"],
                            "fstype": "C",
                        }
                    },
                )

        with patch.dict(mount.__grains__, {"os": "Solaris", "kernel": "SunOS"}):
            mock = MagicMock(return_value="A * B * C D/E/F")
            with patch.dict(mount.__salt__, {"cmd.run_stdout": mock}):
                self.assertEqual(
                    mount.active(),
                    {"B": {"device": "A", "opts": ["D", "E", "F"], "fstype": "C"}},
                )

        with patch.dict(mount.__grains__, {"os": "AIX", "kernel": "AIX"}):
            mock = MagicMock(return_value="A * B * C D/E/F")
            with patch.dict(mount.__salt__, {"cmd.run_stdout": mock}):
                self.assertEqual(
                    mount.active(), {"B": {"node": "A", "device": "*", "fstype": "*"}}
                )

        with patch.dict(mount.__grains__, {"os": "OpenBSD", "kernel": "OpenBSD"}):
            mock = MagicMock(return_value={})
            with patch.object(mount, "_active_mounts_openbsd", mock):
                self.assertEqual(mount.active(), {})

        with patch.dict(mount.__grains__, {"os": "MacOS", "kernel": "Darwin"}):
            mock = MagicMock(return_value={})
            with patch.object(mount, "_active_mounts_darwin", mock):
                self.assertEqual(mount.active(), {})

        with patch.dict(mount.__grains__, {"os": "MacOS", "kernel": "Darwin"}):
            mock = MagicMock(return_value={})
            with patch.object(mount, "_active_mountinfo", mock):
                with patch.object(mount, "_active_mounts_darwin", mock):
                    self.assertEqual(mount.active(extended=True), {})

        with patch.dict(mount.__grains__, {"os": "AIX", "kernel": "AIX"}):
            mock = MagicMock(return_value={})
            with patch.object(mount, "_active_mounts_aix", mock):
                self.assertEqual(mount.active(), {})

    def test_fstab(self):
        """
        List the content of the fstab
        """
        mock = MagicMock(return_value=False)
        with patch.object(os.path, "isfile", mock):
            self.assertEqual(mount.fstab(), {})

        file_data = "\n".join(["#", "A B C D,E,F G H"])
        mock = MagicMock(return_value=True)
        with patch.dict(mount.__grains__, {"kernel": ""}), patch.object(
            os.path, "isfile", mock
        ), patch("salt.utils.files.fopen", mock_open(read_data=file_data)):
            fstab = mount.fstab()
            assert fstab == {
                "B": {
                    "device": "A",
                    "dump": "G",
                    "fstype": "C",
                    "opts": ["D", "E", "F"],
                    "pass": "H",
                }
            }, fstab

    def test_vfstab(self):
        """
        List the content of the vfstab
        """
        mock = MagicMock(return_value=False)
        with patch.object(os.path, "isfile", mock):
            self.assertEqual(mount.vfstab(), {})

        file_data = textwrap.dedent(
            """\
            #
            swap        -   /tmp                tmpfs    -   yes    size=2048m
            """
        )
        mock = MagicMock(return_value=True)
        with patch.dict(mount.__grains__, {"kernel": "SunOS"}), patch.object(
            os.path, "isfile", mock
        ), patch("salt.utils.files.fopen", mock_open(read_data=file_data)):
            vfstab = mount.vfstab()
            assert vfstab == {
                "/tmp": {
                    "device": "swap",
                    "device_fsck": "-",
                    "fstype": "tmpfs",
                    "mount_at_boot": "yes",
                    "opts": ["size=2048m"],
                    "pass_fsck": "-",
                }
            }, vfstab

    def test_filesystems(self):
        """
        List the content of the filesystems
        """
        file_data = textwrap.dedent(
            """\
            #

            """
        )
        mock = MagicMock(return_value=True)
        with patch.dict(mount.__grains__, {"os": "AIX", "kernel": "AIX"}), patch.object(
            os.path, "isfile", mock
        ), patch("salt.utils.files.fopen", mock_open(read_data=file_data)):
            self.assertEqual(mount.filesystems(), {})

        file_data = textwrap.dedent(
            """\
            #
            /home:
                    dev             = /dev/hd1
                    vfs             = jfs2
                    log             = /dev/hd8
                    mount           = true
                    check           = true
                    vol             = /home
                    free            = false
                    quota           = no

            """
        )
        mock = MagicMock(return_value=True)
        with patch.dict(mount.__grains__, {"os": "AIX", "kernel": "AIX"}), patch.object(
            os.path, "isfile", mock
        ), patch("salt.utils.files.fopen", mock_open(read_data=file_data)):
            fsyst = mount.filesystems()
            test_fsyst = {
                "/home": {
                    "dev": "/dev/hd1",
                    "vfs": "jfs2",
                    "log": "/dev/hd8",
                    "mount": "true",
                    "check": "true",
                    "vol": "/home",
                    "free": "false",
                    "quota": "no",
                }
            }
            self.assertEqual(test_fsyst, fsyst)

    def test_rm_fstab(self):
        """
        Remove the mount point from the fstab
        """
        mock_fstab = MagicMock(return_value={})
        with patch.dict(mount.__grains__, {"kernel": ""}):
            with patch.object(mount, "fstab", mock_fstab):
                with patch("salt.utils.files.fopen", mock_open()):
                    self.assertTrue(mount.rm_fstab("name", "device"))

    def test_set_fstab(self):
        """
        Tests to verify that this mount is represented in the fstab,
        change the mount to match the data passed, or add the mount
        if it is not present.
        """
        mock = MagicMock(return_value=False)
        with patch.object(os.path, "isfile", mock):
            self.assertRaises(CommandExecutionError, mount.set_fstab, "A", "B", "C")

        mock = MagicMock(return_value=True)
        mock_read = MagicMock(side_effect=OSError)
        with patch.object(os.path, "isfile", mock):
            with patch.object(salt.utils.files, "fopen", mock_read):
                self.assertRaises(CommandExecutionError, mount.set_fstab, "A", "B", "C")

        mock = MagicMock(return_value=True)
        with patch.object(os.path, "isfile", mock):
            with patch("salt.utils.files.fopen", mock_open(read_data=MOCK_SHELL_FILE)):
                self.assertEqual(mount.set_fstab("A", "B", "C"), "new")

        mock = MagicMock(return_value=True)
        with patch.object(os.path, "isfile", mock):
            with patch("salt.utils.files.fopen", mock_open(read_data=MOCK_SHELL_FILE)):
                self.assertEqual(
                    mount.set_fstab("B", "A", "C", "D", "F", "G"), "present"
                )

        mock = MagicMock(return_value=True)
        with patch.object(os.path, "isfile", mock):
            with patch("salt.utils.files.fopen", mock_open(read_data=MOCK_SHELL_FILE)):
                self.assertEqual(
                    mount.set_fstab("B", "A", "C", not_change=True), "present"
                )

    def test_rm_automaster(self):
        """
        Remove the mount point from the auto_master
        """
        mock = MagicMock(return_value={})
        with patch.object(mount, "automaster", mock):
            self.assertTrue(mount.rm_automaster("name", "device"))

        mock = MagicMock(return_value={"name": "name"})
        with patch.object(mount, "fstab", mock):
            self.assertTrue(mount.rm_automaster("name", "device"))

    def test_set_automaster(self):
        """
        Verify that this mount is represented in the auto_salt, change the mount
        to match the data passed, or add the mount if it is not present.
        """
        mock = MagicMock(return_value=True)
        with patch.object(os.path, "isfile", mock):
            self.assertRaises(
                CommandExecutionError, mount.set_automaster, "A", "B", "C"
            )

        mock = MagicMock(return_value=True)
        mock_read = MagicMock(side_effect=OSError)
        with patch.object(os.path, "isfile", mock):
            with patch.object(salt.utils.files, "fopen", mock_read):
                self.assertRaises(
                    CommandExecutionError, mount.set_automaster, "A", "B", "C"
                )

        mock = MagicMock(return_value=True)
        with patch.object(os.path, "isfile", mock):
            with patch("salt.utils.files.fopen", mock_open(read_data=MOCK_SHELL_FILE)):
                self.assertEqual(mount.set_automaster("A", "B", "C"), "new")

        mock = MagicMock(return_value=True)
        with patch.object(os.path, "isfile", mock):
            with patch(
                "salt.utils.files.fopen", mock_open(read_data="/..A -fstype=C,D C:B")
            ):
                self.assertEqual(mount.set_automaster("A", "B", "C", "D"), "present")

        mock = MagicMock(return_value=True)
        with patch.object(os.path, "isfile", mock):
            with patch(
                "salt.utils.files.fopen", mock_open(read_data="/..A -fstype=XX C:B")
            ):
                self.assertEqual(
                    mount.set_automaster("A", "B", "C", "D", not_change=True), "present"
                )

    def test_automaster(self):
        """
        Test the list the contents of the fstab
        """
        self.assertDictEqual(mount.automaster(), {})

    def test_rm_filesystems(self):
        """
        Remove the mount point from the filesystems
        """
        file_data = textwrap.dedent(
            """\
            #

            """
        )
        mock = MagicMock(return_value=True)
        with patch.dict(mount.__grains__, {"os": "AIX", "kernel": "AIX"}), patch.object(
            os.path, "isfile", mock
        ), patch("salt.utils.files.fopen", mock_open(read_data=file_data)):
            self.assertFalse(mount.rm_filesystems("name", "device"))

        file_data = textwrap.dedent(
            """\
            #
            /name:
                    dev             = device
                    vol             = /name

            """
        )

        mock = MagicMock(return_value=True)
        mock_fsyst = MagicMock(return_value=True)
        with patch.dict(mount.__grains__, {"os": "AIX", "kernel": "AIX"}), patch.object(
            os.path, "isfile", mock
        ), patch("salt.utils.files.fopen", mock_open(read_data=file_data)):
            self.assertTrue(mount.rm_filesystems("/name", "device"))

    def test_set_filesystems(self):
        """
        Tests to verify that this mount is represented in the filesystems,
        change the mount to match the data passed, or add the mount
        if it is not present.
        """
        mock = MagicMock(return_value=False)
        with patch.dict(mount.__grains__, {"os": "AIX", "kernel": "AIX"}):
            with patch.object(os.path, "isfile", mock):
                self.assertRaises(
                    CommandExecutionError, mount.set_filesystems, "A", "B", "C"
                )

            mock_read = MagicMock(side_effect=OSError)
            with patch.object(os.path, "isfile", mock):
                with patch.object(salt.utils.files, "fopen", mock_read):
                    self.assertRaises(
                        CommandExecutionError, mount.set_filesystems, "A", "B", "C"
                    )

    def test_mount(self):
        """
        Mount a device
        """
        with patch.dict(mount.__grains__, {"os": "MacOS"}):
            mock = MagicMock(return_value=True)
            with patch.object(os.path, "exists", mock):
                mock = MagicMock(return_value=None)
                with patch.dict(mount.__salt__, {"file.mkdir": None}):
                    mock = MagicMock(return_value={"retcode": True, "stderr": True})
                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device"))
                        mock.assert_called_with(
                            "mount  device name ", python_shell=False, runas=None
                        )

                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device", fstype="fstype"))
                        mock.assert_called_with(
                            "mount  -t fstype device name ",
                            python_shell=False,
                            runas=None,
                        )

                    mock = MagicMock(return_value={"retcode": False, "stderr": False})
                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device"))

        with patch.dict(mount.__grains__, {"os": "AIX"}):
            mock = MagicMock(return_value=True)
            with patch.object(os.path, "exists", mock):
                mock = MagicMock(return_value=None)
                with patch.dict(mount.__salt__, {"file.mkdir": None}):
                    mock = MagicMock(return_value={"retcode": True, "stderr": True})
                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device"))
                        mock.assert_called_with(
                            "mount  device name ", python_shell=False, runas=None
                        )

                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device", fstype="fstype"))
                        mock.assert_called_with(
                            "mount  -v fstype device name ",
                            python_shell=False,
                            runas=None,
                        )

                    mock = MagicMock(return_value={"retcode": False, "stderr": False})
                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device"))

        with patch.dict(mount.__grains__, {"os": "Linux"}):
            mock = MagicMock(return_value=True)
            with patch.object(os.path, "exists", mock):
                mock = MagicMock(return_value=None)
                with patch.dict(mount.__salt__, {"file.mkdir": None}):
                    mock = MagicMock(return_value={"retcode": True, "stderr": True})
                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device"))
                        mock.assert_called_with(
                            "mount -o defaults device name ",
                            python_shell=False,
                            runas=None,
                        )

                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device", fstype="fstype"))
                        mock.assert_called_with(
                            "mount -o defaults -t fstype device name ",
                            python_shell=False,
                            runas=None,
                        )

                    mock = MagicMock(return_value={"retcode": False, "stderr": False})
                    with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                        self.assertTrue(mount.mount("name", "device"))

    def test_remount_non_mounted(self):
        """
        Attempt to remount a device, if the device is not already mounted, mount
        is called
        """
        with patch.dict(mount.__grains__, {"os": "MacOS"}):
            mock = MagicMock(return_value=[])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value=True)
                with patch.object(mount, "mount", mock):
                    self.assertTrue(mount.remount("name", "device"))

        with patch.dict(mount.__grains__, {"os": "AIX"}):
            mock = MagicMock(return_value=[])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value=True)
                with patch.object(mount, "mount", mock):
                    self.assertTrue(mount.remount("name", "device"))

        with patch.dict(mount.__grains__, {"os": "Linux"}):
            mock = MagicMock(return_value=[])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value=True)
                with patch.object(mount, "mount", mock):
                    self.assertTrue(mount.remount("name", "device"))

    def test_remount_already_mounted_no_fstype(self):
        """
        Attempt to remount a device already mounted that do not provides
        fstype
        """
        with patch.dict(mount.__grains__, {"os": "MacOS"}):
            mock = MagicMock(return_value=["name"])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value={"retcode": 0})
                with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                    self.assertTrue(mount.remount("name", "device"))
                    mock.assert_called_with(
                        "mount -u -o noowners device name ",
                        python_shell=False,
                        runas=None,
                    )

        with patch.dict(mount.__grains__, {"os": "AIX"}):
            mock = MagicMock(return_value=["name"])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value={"retcode": 0})
                with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                    self.assertTrue(mount.remount("name", "device"))
                    mock.assert_called_with(
                        "mount -o remount device name ", python_shell=False, runas=None
                    )

        with patch.dict(mount.__grains__, {"os": "Linux"}):
            mock = MagicMock(return_value=["name"])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value={"retcode": 0})
                with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                    self.assertTrue(mount.remount("name", "device"))
                    mock.assert_called_with(
                        "mount -o defaults,remount device name ",
                        python_shell=False,
                        runas=None,
                    )

    def test_remount_already_mounted_with_fstype(self):
        """
        Attempt to remount a device already mounted that do not provides
        fstype
        """
        with patch.dict(mount.__grains__, {"os": "MacOS"}):
            mock = MagicMock(return_value=["name"])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value={"retcode": 0})
                with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                    self.assertTrue(mount.remount("name", "device", fstype="type"))
                    mock.assert_called_with(
                        "mount -u -o noowners -t type device name ",
                        python_shell=False,
                        runas=None,
                    )

        with patch.dict(mount.__grains__, {"os": "AIX"}):
            mock = MagicMock(return_value=["name"])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value={"retcode": 0})
                with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                    self.assertTrue(mount.remount("name", "device", fstype="type"))
                    mock.assert_called_with(
                        "mount -o remount -v type device name ",
                        python_shell=False,
                        runas=None,
                    )

        with patch.dict(mount.__grains__, {"os": "Linux"}):
            mock = MagicMock(return_value=["name"])
            with patch.object(mount, "active", mock):
                mock = MagicMock(return_value={"retcode": 0})
                with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                    self.assertTrue(mount.remount("name", "device", fstype="type"))
                    mock.assert_called_with(
                        "mount -o defaults,remount -t type device name ",
                        python_shell=False,
                        runas=None,
                    )

    def test_umount(self):
        """
        Attempt to unmount a device by specifying the directory it is
        mounted on
        """
        mock = MagicMock(return_value={})
        with patch.object(mount, "active", mock):
            self.assertEqual(
                mount.umount("name"), "name does not have anything mounted"
            )

        mock = MagicMock(return_value={"name": "name"})
        with patch.object(mount, "active", mock):
            mock = MagicMock(return_value={"retcode": True, "stderr": True})
            with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                self.assertTrue(mount.umount("name"))

            mock = MagicMock(return_value={"retcode": False})
            with patch.dict(mount.__salt__, {"cmd.run_all": mock}):
                self.assertTrue(mount.umount("name"))

        # Test unmounting with guestfs util
        mock = MagicMock()
        with patch.dict(mount.__salt__, {"guestfs.umount": mock}):
            mount.umount("/mountpoint", device="/path/to/my.qcow", util="guestfs")
            mock.assert_called_once_with("/mountpoint", disk="/path/to/my.qcow")

    def test_is_fuse_exec(self):
        """
        Returns true if the command passed is a fuse mountable application
        """
        with patch.object(salt.utils.path, "which", return_value=None):
            self.assertFalse(mount.is_fuse_exec("cmd"))

        def _ldd_side_effect(cmd, *args, **kwargs):
            """
            Neither of these are full ldd output, but what is_fuse_exec is
            looking for is 'libfuse' in the ldd output, so these examples
            should be sufficient enough to test both the True and False cases.
            """
            return {
                "ldd cmd1": textwrap.dedent(
                    """\
                    linux-vdso.so.1 (0x00007ffeaf5fb000)
                    libfuse3.so.3 => /usr/lib/libfuse3.so.3 (0x00007f91e66ac000)
                    """
                ),
                "ldd cmd2": textwrap.dedent(
                    """\
                    linux-vdso.so.1 (0x00007ffeaf5fb000)
                    """
                ),
            }[cmd]

        which_mock = MagicMock(side_effect=lambda x: x)
        ldd_mock = MagicMock(side_effect=_ldd_side_effect)
        with patch.object(salt.utils.path, "which", which_mock):
            with patch.dict(mount.__salt__, {"cmd.run": _ldd_side_effect}):
                self.assertTrue(mount.is_fuse_exec("cmd1"))
                self.assertFalse(mount.is_fuse_exec("cmd2"))

    def test_swaps(self):
        """
        Return a dict containing information on active swap
        """
        file_data = textwrap.dedent(
            """\
            Filename Type Size Used Priority
            /dev/sda1 partition 31249404 4100 -1
            """
        )
        with patch.dict(mount.__grains__, {"os": "", "kernel": ""}):
            with patch("salt.utils.files.fopen", mock_open(read_data=file_data)):
                swaps = mount.swaps()
                assert swaps == {
                    "/dev/sda1": {
                        "priority": "-1",
                        "size": "31249404",
                        "type": "partition",
                        "used": "4100",
                    }
                }, swaps

        file_data = textwrap.dedent(
            """\
            Device Size Used Unknown Unknown Priority
            /dev/sda1 31249404 4100 unknown unknown -1
            """
        )
        mock = MagicMock(return_value=file_data)
        with patch.dict(
            mount.__grains__, {"os": "OpenBSD", "kernel": "OpenBSD"}
        ), patch.dict(mount.__salt__, {"cmd.run_stdout": mock}):
            swaps = mount.swaps()
            assert swaps == {
                "/dev/sda1": {
                    "priority": "-1",
                    "size": "31249404",
                    "type": "partition",
                    "used": "4100",
                }
            }, swaps

        file_data = textwrap.dedent(
            """\
            device              maj,min        total       free
            /dev/hd6              10,  2     11776MB     11765MB
            """
        )
        mock = MagicMock(return_value=file_data)
        with patch.dict(mount.__grains__, {"os": "AIX", "kernel": "AIX"}), patch.dict(
            mount.__salt__, {"cmd.run_stdout": mock}
        ):
            swaps = mount.swaps()
            assert swaps == {
                "/dev/hd6": {
                    "priority": "-",
                    "size": 12058624,
                    "type": "device",
                    "used": 11264,
                }
            }, swaps

    def test_swapon(self):
        """
        Activate a swap disk
        """
        mock = MagicMock(return_value={"name": "name"})
        with patch.dict(mount.__grains__, {"kernel": ""}):
            with patch.object(mount, "swaps", mock):
                self.assertEqual(mount.swapon("name"), {"stats": "name", "new": False})

        mock = MagicMock(return_value={})
        with patch.dict(mount.__grains__, {"kernel": ""}):
            with patch.object(mount, "swaps", mock):
                mock = MagicMock(return_value=None)
                with patch.dict(mount.__salt__, {"cmd.run": mock}):
                    self.assertEqual(mount.swapon("name", False), {})

        mock = MagicMock(side_effect=[{}, {"name": "name"}])
        with patch.dict(mount.__grains__, {"kernel": ""}):
            with patch.object(mount, "swaps", mock):
                mock = MagicMock(return_value=None)
                with patch.dict(mount.__salt__, {"cmd.run": mock}):
                    self.assertEqual(
                        mount.swapon("name"), {"stats": "name", "new": True}
                    )
        ## effects of AIX
        mock = MagicMock(return_value={"name": "name"})
        with patch.dict(mount.__grains__, {"kernel": "AIX"}):
            with patch.object(mount, "swaps", mock):
                self.assertEqual(mount.swapon("name"), {"stats": "name", "new": False})

        mock = MagicMock(return_value={})
        with patch.dict(mount.__grains__, {"kernel": "AIX"}):
            with patch.object(mount, "swaps", mock):
                mock = MagicMock(return_value=None)
                with patch.dict(mount.__salt__, {"cmd.run": mock}):
                    self.assertEqual(mount.swapon("name", False), {})

        mock = MagicMock(side_effect=[{}, {"name": "name"}])
        with patch.dict(mount.__grains__, {"kernel": "AIX"}):
            with patch.object(mount, "swaps", mock):
                mock = MagicMock(return_value=None)
                with patch.dict(mount.__salt__, {"cmd.run": mock}):
                    self.assertEqual(
                        mount.swapon("name"), {"stats": "name", "new": True}
                    )

    def test_swapoff(self):
        """
        Deactivate a named swap mount
        """
        mock = MagicMock(return_value={})
        with patch.dict(mount.__grains__, {"kernel": ""}):
            with patch.object(mount, "swaps", mock):
                self.assertEqual(mount.swapoff("name"), None)

        mock = MagicMock(return_value={"name": "name"})
        with patch.dict(mount.__grains__, {"kernel": ""}):
            with patch.object(mount, "swaps", mock):
                with patch.dict(mount.__grains__, {"os": "test"}):
                    mock = MagicMock(return_value=None)
                    with patch.dict(mount.__salt__, {"cmd.run": mock}):
                        self.assertFalse(mount.swapoff("name"))

        mock = MagicMock(side_effect=[{"name": "name"}, {}])
        with patch.dict(mount.__grains__, {"kernel": ""}):
            with patch.object(mount, "swaps", mock):
                with patch.dict(mount.__grains__, {"os": "test"}):
                    mock = MagicMock(return_value=None)
                    with patch.dict(mount.__salt__, {"cmd.run": mock}):
                        self.assertTrue(mount.swapoff("name"))

        # check on AIX
        mock = MagicMock(return_value={})
        with patch.dict(mount.__grains__, {"kernel": "AIX"}):
            with patch.object(mount, "swaps", mock):
                self.assertEqual(mount.swapoff("name"), None)

        mock = MagicMock(return_value={"name": "name"})
        with patch.dict(mount.__grains__, {"kernel": "AIX"}):
            with patch.object(mount, "swaps", mock):
                with patch.dict(mount.__grains__, {"os": "test"}):
                    mock = MagicMock(return_value=None)
                    with patch.dict(mount.__salt__, {"cmd.run": mock}):
                        self.assertFalse(mount.swapoff("name"))

        mock = MagicMock(side_effect=[{"name": "name"}, {}])
        with patch.dict(mount.__grains__, {"kernel": "AIX"}):
            with patch.object(mount, "swaps", mock):
                with patch.dict(mount.__grains__, {"os": "test"}):
                    mock = MagicMock(return_value=None)
                    with patch.dict(mount.__salt__, {"cmd.run": mock}):
                        self.assertTrue(mount.swapoff("name"))

    def test_is_mounted(self):
        """
        Provide information if the path is mounted
        """
        mock = MagicMock(return_value={})
        with patch.object(mount, "active", mock), patch.dict(
            mount.__grains__, {"kernel": ""}
        ):
            self.assertFalse(mount.is_mounted("name"))

        mock = MagicMock(return_value={"name": "name"})
        with patch.object(mount, "active", mock), patch.dict(
            mount.__grains__, {"kernel": ""}
        ):
            self.assertTrue(mount.is_mounted("name"))



import errno
import logging
import threading
import time

import pytest
import salt.exceptions
import salt.payload
import zmq

log = logging.getLogger(__name__)


@pytest.fixture
def echo_port():
    yield 8845


class EchoServer:
    def __init__(self, port=8845):
        self.thread_running = threading.Event()
        self.thread_running.set()
        self.port = port
        self.thread = threading.Thread(
            target=self.echo_server, args=(port, self.thread_running)
        )

    def start(self):
        self.thread.start()
        time.sleep(1)

    def stop(self):
        self.thread_running.clear()
        self.thread.join()

    @staticmethod
    def echo_server(port, event):
        """
        A server that echos the message sent to it over zmq

        Optional "sleep" can be sent to delay response
        """
        context = zmq.Context()
        socket = context.socket(zmq.REP)
        socket.bind("tcp://*:{}".format(port))
        while event.is_set():
            try:
                #  Wait for next request from client
                message = socket.recv(zmq.NOBLOCK)
                msg_deserialized = salt.payload.loads(message)
                log.info("Echo server received message: %s", msg_deserialized)
                if isinstance(msg_deserialized["load"], dict) and msg_deserialized[
                    "load"
                ].get("sleep"):
                    log.info(
                        "Test echo server sleeping for %s seconds",
                        msg_deserialized["load"]["sleep"],
                    )
                    time.sleep(msg_deserialized["load"]["sleep"])
                socket.send(message)
            except zmq.ZMQError as exc:
                if exc.errno == errno.EAGAIN:
                    continue
                raise


@pytest.fixture
def echo_server(echo_port):
    server = EchoServer()
    server.start()
    try:
        yield server
    finally:
        server.stop()


@pytest.fixture
def sreq(echo_port):
    yield salt.payload.SREQ("tcp://127.0.0.1:{}".format(echo_port))


@pytest.mark.slow_test
def test_send_auto(sreq, echo_server):
    """
    Test creation, send/rect
    """
    # check default of empty load and enc clear
    assert sreq.send_auto({}) == {"enc": "clear", "load": {}}

    # check that the load always gets passed
    assert sreq.send_auto({"load": "foo"}) == {"load": "foo", "enc": "clear"}


@pytest.mark.slow_test
def test_send(sreq, echo_server):
    assert sreq.send("clear", "foo") == {"enc": "clear", "load": "foo"}


@pytest.mark.skip("Disabled until we can figure out how to make this more reliable.")
def test_timeout(sreq, echo_server):
    """
    Test SREQ Timeouts
    """
    # client-side timeout
    start = time.time()
    # This is a try/except instead of an assertRaises because of a possible
    # subtle bug in zmq wherein a timeout=0 actually executes a single poll
    # before the timeout is reached.
    log.info("Sending tries=0, timeout=0")
    try:
        sreq.send("clear", "foo", tries=0, timeout=0)
    except salt.exceptions.SaltReqTimeoutError:
        pass
    assert time.time() - start < 1  # ensure we didn't wait

    # server-side timeout
    log.info("Sending tries=1, timeout=1")
    start = time.time()
    with pytest.raises(salt.exceptions.SaltReqTimeoutError):
        sreq.send("clear", {"sleep": 2}, tries=1, timeout=1)
    assert time.time() - start >= 1  # ensure we actually tried once (1s)

    # server-side timeout with retries
    log.info("Sending tries=2, timeout=1")
    start = time.time()
    with pytest.raises(salt.exceptions.SaltReqTimeoutError):
        sreq.send("clear", {"sleep": 2}, tries=2, timeout=1)
    assert time.time() - start >= 2  # ensure we actually tried twice (2s)

    # test a regular send afterwards (to make sure sockets aren't in a twist
    log.info("Sending regular send")
    assert sreq.send("clear", "foo") == {"enc": "clear", "load": "foo"}


@pytest.mark.slow_test
def test_destroy(sreq, echo_server):
    """
    Test the __del__ capabilities
    """
    # ensure no exceptions when we go to destroy the sreq, since __del__
    # swallows exceptions, we have to call destroy directly
    sreq.destroy()



import logging
import random
import string

import pytest
import salt.config
import salt.loader
import salt.states.boto_iot as boto_iot
from tests.support.mock import MagicMock, patch

boto = pytest.importorskip("boto")
boto3 = pytest.importorskip("boto3", "1.2.1")
botocore = pytest.importorskip("botocore", "1.4.41")

log = logging.getLogger(__name__)


class GlobalConfig:
    region = "us-east-1"
    access_key = "GKTADJGHEIQSXMKKRBJ08H"
    secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
    conn_parameters = {
        "region": region,
        "key": access_key,
        "keyid": secret_key,
        "profile": {},
    }
    error_message = (
        "An error occurred (101) when calling the {0} operation: Test-defined error"
    )
    not_found_error = botocore.exceptions.ClientError(
        {
            "Error": {
                "Code": "ResourceNotFoundException",
                "Message": "Test-defined error",
            }
        },
        "msg",
    )
    topic_rule_not_found_error = botocore.exceptions.ClientError(
        {"Error": {"Code": "UnauthorizedException", "Message": "Test-defined error"}},
        "msg",
    )
    error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
    policy_ret = dict(
        policyName="testpolicy",
        policyDocument=(
            '{"Version": "2012-10-17", "Statement": [{"Action": ["iot:Publish"],'
            ' "Resource": ["*"], "Effect": "Allow"}]}'
        ),
        policyArn="arn:aws:iot:us-east-1:123456:policy/my_policy",
        policyVersionId=1,
        defaultVersionId=1,
    )
    topic_rule_ret = dict(
        ruleName="testrule",
        sql="SELECT * FROM 'iot/test'",
        description="topic rule description",
        createdAt="1970-01-01",
        actions=[{"iot": {"functionArn": "arn:aws:::function"}}],
        ruleDisabled=True,
    )
    principal = "arn:aws:iot:us-east-1:1234:cert/21fc104aaaf6043f5756c1b57bda84ea8395904c43f28517799b19e4c42514"

    thing_type_name = "test_thing_type"
    thing_type_desc = "test_thing_type_desc"
    thing_type_attr_1 = "test_thing_type_search_attr_1"
    thing_type_ret = dict(
        thingTypeName=thing_type_name,
        thingTypeProperties=dict(
            thingTypeDescription=thing_type_desc,
            searchableAttributes=[thing_type_attr_1],
        ),
        thingTypeMetadata=dict(
            deprecated=False, creationDate="2010-08-01 15:54:49.699000+00:00"
        ),
    )
    deprecated_thing_type_ret = dict(
        thingTypeName=thing_type_name,
        thingTypeProperties=dict(
            thingTypeDescription=thing_type_desc,
            searchableAttributes=[thing_type_attr_1],
        ),
        thingTypeMetadata=dict(
            deprecated=True,
            creationDate="2010-08-01 15:54:49.699000+00:00",
            deprecationDate="2010-08-02 15:54:49.699000+00:00",
        ),
    )
    thing_type_arn = "test_thing_type_arn"
    create_thing_type_ret = dict(
        thingTypeName=thing_type_name, thingTypeArn=thing_type_arn
    )


@pytest.fixture
def configure_loader_modules():
    opts = salt.config.DEFAULT_MINION_OPTS.copy()
    opts["grains"] = salt.loader.grains(opts)
    ctx = {}
    utils = salt.loader.utils(
        opts,
        whitelist=["boto3", "args", "systemd", "path", "platform"],
        context=ctx,
    )
    serializers = salt.loader.serializers(opts)
    funcs = funcs = salt.loader.minion_mods(
        opts, context=ctx, utils=utils, whitelist=["boto_iot"]
    )
    salt_states = salt.loader.states(
        opts=opts,
        functions=funcs,
        utils=utils,
        whitelist=["boto_iot"],
        serializers=serializers,
    )
    return {
        boto_iot: {
            "__opts__": opts,
            "__salt__": funcs,
            "__utils__": utils,
            "__states__": salt_states,
            "__serializers__": serializers,
        }
    }


def test_present_when_thing_type_does_not_exist():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.describe_thing_type.side_effect = [
        GlobalConfig.not_found_error,
        GlobalConfig.thing_type_ret,
    ]
    conn.create_thing_type.return_value = GlobalConfig.create_thing_type_ret
    result = boto_iot.__states__["boto_iot.thing_type_present"](
        "thing type present",
        thingTypeName=GlobalConfig.thing_type_name,
        thingTypeDescription=GlobalConfig.thing_type_desc,
        searchableAttributesList=[GlobalConfig.thing_type_attr_1],
        **GlobalConfig.conn_parameters
    )
    assert result["result"]
    assert (
        result["changes"]["new"]["thing_type"]["thingTypeName"]
        == GlobalConfig.thing_type_name
    )


def test_present_when_thing_type_exists():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.describe_thing_type.return_value = GlobalConfig.thing_type_ret
    result = boto_iot.__states__["boto_iot.thing_type_present"](
        "thing type present",
        thingTypeName=GlobalConfig.thing_type_name,
        thingTypeDescription=GlobalConfig.thing_type_desc,
        searchableAttributesList=[GlobalConfig.thing_type_attr_1],
        **GlobalConfig.conn_parameters
    )
    assert result["result"]
    assert result["changes"] == {}
    assert conn.create_thing_type.call_count == 0


def test_present_with_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.describe_thing_type.side_effect = [
        GlobalConfig.not_found_error,
        GlobalConfig.thing_type_ret,
    ]
    conn.create_thing_type.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "create_thing_type"
    )
    result = boto_iot.__states__["boto_iot.thing_type_present"](
        "thing type present",
        thingTypeName=GlobalConfig.thing_type_name,
        thingTypeDescription=GlobalConfig.thing_type_desc,
        searchableAttributesList=[GlobalConfig.thing_type_attr_1],
        **GlobalConfig.conn_parameters
    )
    assert not result["result"]
    assert "An error occurred" in result["comment"]


def test_absent_when_thing_type_does_not_exist():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.describe_thing_type.side_effect = GlobalConfig.not_found_error
    result = boto_iot.__states__["boto_iot.thing_type_absent"](
        "test", "mythingtype", **GlobalConfig.conn_parameters
    )
    assert result["result"]
    assert result["changes"] == {}


@pytest.mark.slow_test
def test_absent_when_thing_type_exists():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.describe_thing_type.return_value = GlobalConfig.deprecated_thing_type_ret
    result = boto_iot.__states__["boto_iot.thing_type_absent"](
        "test", GlobalConfig.thing_type_name, **GlobalConfig.conn_parameters
    )
    assert result["result"]
    assert result["changes"]["new"]["thing_type"] is None
    assert conn.deprecate_thing_type.call_count == 0


def test_absent_with_deprecate_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.describe_thing_type.return_value = GlobalConfig.thing_type_ret
    conn.deprecate_thing_type.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "deprecate_thing_type"
    )
    result = boto_iot.__states__["boto_iot.thing_type_absent"](
        "test", GlobalConfig.thing_type_name, **GlobalConfig.conn_parameters
    )
    assert not result["result"]
    assert "An error occurred" in result["comment"]
    assert "deprecate_thing_type" in result["comment"]
    assert conn.delete_thing_type.call_count == 0


def test_absent_with_delete_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.describe_thing_type.return_value = GlobalConfig.deprecated_thing_type_ret
    conn.delete_thing_type.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "delete_thing_type"
    )
    result = boto_iot.__states__["boto_iot.thing_type_absent"](
        "test", GlobalConfig.thing_type_name, **GlobalConfig.conn_parameters
    )
    assert not result["result"]
    assert "An error occurred" in result["comment"]
    assert "delete_thing_type" in result["comment"]
    assert conn.deprecate_thing_type.call_count == 0


def test_present_when_policy_does_not_exist():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_policy.side_effect = [
        GlobalConfig.not_found_error,
        GlobalConfig.policy_ret,
    ]
    conn.create_policy.return_value = GlobalConfig.policy_ret
    result = boto_iot.__states__["boto_iot.policy_present"](
        "policy present",
        policyName=GlobalConfig.policy_ret["policyName"],
        policyDocument=GlobalConfig.policy_ret["policyDocument"],
    )

    assert result["result"]
    assert (
        result["changes"]["new"]["policy"]["policyName"]
        == GlobalConfig.policy_ret["policyName"]
    )


def test_present_when_policy_exists():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_policy.return_value = GlobalConfig.policy_ret
    conn.create_policy_version.return_value = GlobalConfig.policy_ret
    result = boto_iot.__states__["boto_iot.policy_present"](
        "policy present",
        policyName=GlobalConfig.policy_ret["policyName"],
        policyDocument=GlobalConfig.policy_ret["policyDocument"],
    )
    assert result["result"]
    assert result["changes"] == {}


def test_present_again_with_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_policy.side_effect = [
        GlobalConfig.not_found_error,
        GlobalConfig.policy_ret,
    ]
    conn.create_policy.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "create_policy"
    )
    result = boto_iot.__states__["boto_iot.policy_present"](
        "policy present",
        policyName=GlobalConfig.policy_ret["policyName"],
        policyDocument=GlobalConfig.policy_ret["policyDocument"],
    )
    assert not result["result"]
    assert "An error occurred" in result["comment"]


def test_absent_when_policy_does_not_exist():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_policy.side_effect = GlobalConfig.not_found_error
    result = boto_iot.__states__["boto_iot.policy_absent"]("test", "mypolicy")
    assert result["result"]
    assert result["changes"] == {}


def test_absent_when_policy_exists():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_policy.return_value = GlobalConfig.policy_ret
    conn.list_policy_versions.return_value = {"policyVersions": []}
    result = boto_iot.__states__["boto_iot.policy_absent"](
        "test", GlobalConfig.policy_ret["policyName"]
    )
    assert result["result"]
    assert result["changes"]["new"]["policy"] is None


def test_absent_with_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_policy.return_value = GlobalConfig.policy_ret
    conn.list_policy_versions.return_value = {"policyVersions": []}
    conn.delete_policy.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "delete_policy"
    )
    result = boto_iot.__states__["boto_iot.policy_absent"](
        "test", GlobalConfig.policy_ret["policyName"]
    )
    assert not result["result"]
    assert "An error occurred" in result["comment"]


def test_attached_when_policy_not_attached():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.list_principal_policies.return_value = {"policies": []}
    result = boto_iot.__states__["boto_iot.policy_attached"](
        "test", "myfunc", GlobalConfig.principal
    )
    assert result["result"]
    assert result["changes"]["new"]["attached"]


def test_attached_when_policy_attached():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.list_principal_policies.return_value = {"policies": [GlobalConfig.policy_ret]}
    result = boto_iot.__states__["boto_iot.policy_attached"](
        "test", GlobalConfig.policy_ret["policyName"], GlobalConfig.principal
    )
    assert result["result"]
    assert result["changes"] == {}


def test_attached_with_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.list_principal_policies.return_value = {"policies": []}
    conn.attach_principal_policy.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "attach_principal_policy"
    )
    result = boto_iot.__states__["boto_iot.policy_attached"](
        "test", GlobalConfig.policy_ret["policyName"], GlobalConfig.principal
    )
    assert not result["result"]
    assert result["changes"] == {}


def test_detached_when_policy_not_detached():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.list_principal_policies.return_value = {"policies": [GlobalConfig.policy_ret]}
    result = boto_iot.__states__["boto_iot.policy_detached"](
        "test", GlobalConfig.policy_ret["policyName"], GlobalConfig.principal
    )
    assert result["result"]
    log.warning(result)
    assert not result["changes"]["new"]["attached"]


def test_detached_when_policy_detached():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.list_principal_policies.return_value = {"policies": []}
    result = boto_iot.__states__["boto_iot.policy_detached"](
        "test", GlobalConfig.policy_ret["policyName"], GlobalConfig.principal
    )
    assert result["result"]
    assert result["changes"] == {}


def test_detached_with_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.list_principal_policies.return_value = {"policies": [GlobalConfig.policy_ret]}
    conn.detach_principal_policy.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "detach_principal_policy"
    )
    result = boto_iot.__states__["boto_iot.policy_detached"](
        "test", GlobalConfig.policy_ret["policyName"], GlobalConfig.principal
    )
    assert not result["result"]
    assert result["changes"] == {}


def test_present_when_topic_rule_does_not_exist():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_topic_rule.side_effect = [
        GlobalConfig.topic_rule_not_found_error,
        {"rule": GlobalConfig.topic_rule_ret},
    ]
    conn.create_topic_rule.return_value = {"created": True}
    result = boto_iot.__states__["boto_iot.topic_rule_present"](
        "topic rule present",
        ruleName=GlobalConfig.topic_rule_ret["ruleName"],
        sql=GlobalConfig.topic_rule_ret["sql"],
        description=GlobalConfig.topic_rule_ret["description"],
        actions=GlobalConfig.topic_rule_ret["actions"],
        ruleDisabled=GlobalConfig.topic_rule_ret["ruleDisabled"],
    )

    assert result["result"]
    assert (
        result["changes"]["new"]["rule"]["ruleName"]
        == GlobalConfig.topic_rule_ret["ruleName"]
    )


def test_present_when_next_policy_exists():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_topic_rule.return_value = {"rule": GlobalConfig.topic_rule_ret}
    conn.create_topic_rule.return_value = {"created": True}
    result = boto_iot.__states__["boto_iot.topic_rule_present"](
        "topic rule present",
        ruleName=GlobalConfig.topic_rule_ret["ruleName"],
        sql=GlobalConfig.topic_rule_ret["sql"],
        description=GlobalConfig.topic_rule_ret["description"],
        actions=GlobalConfig.topic_rule_ret["actions"],
        ruleDisabled=GlobalConfig.topic_rule_ret["ruleDisabled"],
    )
    assert result["result"]
    assert result["changes"] == {}


def test_present_next_with_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_topic_rule.side_effect = [
        GlobalConfig.topic_rule_not_found_error,
        {"rule": GlobalConfig.topic_rule_ret},
    ]
    conn.create_topic_rule.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "create_topic_rule"
    )
    result = boto_iot.__states__["boto_iot.topic_rule_present"](
        "topic rule present",
        ruleName=GlobalConfig.topic_rule_ret["ruleName"],
        sql=GlobalConfig.topic_rule_ret["sql"],
        description=GlobalConfig.topic_rule_ret["description"],
        actions=GlobalConfig.topic_rule_ret["actions"],
        ruleDisabled=GlobalConfig.topic_rule_ret["ruleDisabled"],
    )
    assert not result["result"]
    assert "An error occurred" in result["comment"]


def test_absent_when_topic_rule_does_not_exist():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_topic_rule.side_effect = GlobalConfig.topic_rule_not_found_error
    result = boto_iot.__states__["boto_iot.topic_rule_absent"]("test", "myrule")
    assert result["result"]
    assert result["changes"] == {}


def test_absent_when_topic_rule_exists():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_topic_rule.return_value = GlobalConfig.topic_rule_ret
    result = boto_iot.__states__["boto_iot.topic_rule_absent"](
        "test", GlobalConfig.topic_rule_ret["ruleName"]
    )
    assert result["result"]
    assert result["changes"]["new"]["rule"] is None


def test_absent_next_with_failure():
    GlobalConfig.conn_parameters["key"] = "".join(
        random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
    )
    patcher = patch("boto3.session.Session")
    mock_session = patcher.start()
    session_instance = mock_session.return_value
    conn = MagicMock()
    session_instance.client.return_value = conn
    conn.get_topic_rule.return_value = GlobalConfig.topic_rule_ret
    conn.delete_topic_rule.side_effect = botocore.exceptions.ClientError(
        GlobalConfig.error_content, "delete_topic_rule"
    )
    result = boto_iot.__states__["boto_iot.topic_rule_absent"](
        "test", GlobalConfig.topic_rule_ret["ruleName"]
    )
    assert not result["result"]
    assert "An error occurred" in result["comment"]



"""
    :codeauthor: `Anthony Shaw <anthonyshaw@apache.org>`

    tests.pytests.unit.cloud.clouds.test_gce
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

import collections

import pytest
from salt.cloud.clouds import gce
from salt.exceptions import SaltCloudSystemExit
from salt.utils.versions import LooseVersion
from tests.support.mock import MagicMock
from tests.support.mock import __version__ as mock_version
from tests.support.mock import patch

VM_NAME = "kings_landing"
DUMMY_TOKEN = {
    "refresh_token": None,
    "client_id": "dany123",
    "client_secret": "lalalalalalala",
    "grant_type": "refresh_token",
}


class DummyGCEConn:
    def __init__(self):
        self.create_node = MagicMock()
        self.ex_create_address = MagicMock(return_value=DummyGCEAddress())

    def __getattr__(self, attr):
        funcs = ["create_node", "ex_create_address"]
        if attr not in funcs:
            # Return back the first thing passed in (i.e. don't call out to get
            # the override value).
            return lambda *args, **kwargs: args[0]


class DummyGCERegion:
    def __init__(self):
        self.name = MagicMock()


class DummyGCEAddress:
    def __init__(self):
        self.id = MagicMock()
        self.name = MagicMock()
        self.address = MagicMock()
        self.region = DummyGCERegion()
        self.driver = MagicMock()
        self.extra = MagicMock()


@pytest.fixture
def configure_loader_modules():

    return {
        gce: {
            "__active_provider_name__": "",
            "__utils__": {
                "cloud.fire_event": MagicMock(),
                "cloud.filter_event": MagicMock(),
            },
            "__opts__": {
                "sock_dir": True,
                "transport": True,
                "providers": {
                    "my-google-cloud": {
                        "gce": {
                            "project": "daenerys-cloud",
                            "service_account_email_address": (
                                "dany@targaryen.westeros.cloud"
                            ),
                            "service_account_private_key": "/home/dany/PRIVKEY.pem",
                            "driver": "gce",
                            "ssh_interface": "public_ips",
                        }
                    }
                },
            },
        }
    }


@pytest.fixture(scope="module")
def location():
    return collections.namedtuple("Location", "name")("chicago")


@pytest.fixture
def config(location):

    return {
        "name": "new",
        "driver": "gce",
        "profile": None,
        "size": 1234,
        "image": "myimage",
        "location": location,
        "ex_network": "mynetwork",
        "ex_subnetwork": "mysubnetwork",
        "ex_tags": "mytags",
        "ex_metadata": "metadata",
    }


@pytest.fixture
def conn():
    return DummyGCEConn()


def test_destroy_call():
    """
    Tests that a SaltCloudSystemExit is raised when trying to call destroy
    with --function or -f.
    """
    pytest.raises(SaltCloudSystemExit, gce.destroy, vm_name=VM_NAME, call="function")


@pytest.mark.skipif(
    gce.HAS_LIBCLOUD is False, reason="apache-libcloud is not installed"
)
def test_fail_virtual_libcloud_version_too_old():
    # Missing deps
    with patch("libcloud.__version__", "2.4.0"):
        v = gce.__virtual__()
        assert v == (False, "The salt-cloud GCE driver requires apache-libcloud>=2.5.0")


@pytest.mark.skipif(
    gce.HAS_LIBCLOUD is False, reason="apache-libcloud is not installed"
)
def test_fail_virtual_missing_deps():
    # Missing deps
    with patch("salt.config.check_driver_dependencies", return_value=False):
        v = gce.__virtual__()
        assert v is False


@pytest.mark.skipif(
    gce.HAS_LIBCLOUD is False, reason="apache-libcloud is not installed"
)
def test_fail_virtual_deps_missing_config():
    with patch("salt.config.check_driver_dependencies", return_value=True), patch(
        "salt.config.is_provider_configured", return_value=False
    ):
        v = gce.__virtual__()
        assert v is False


def test_import():
    """
    Test that the module picks up installed deps
    """
    with patch("salt.config.check_driver_dependencies", return_value=True) as p:
        get_deps = gce.get_dependencies()
        assert get_deps is True
        if LooseVersion(mock_version) >= LooseVersion("2.0.0"):
            p.assert_called_once()


def test_provider_matches():
    """
    Test that the first configured instance of a gce driver is matched
    """
    p = gce.get_configured_provider()
    assert p is not None


def test_request_instance_with_accelerator(config, location, conn):
    """
    Test requesting an instance with GCE accelerators
    """

    config.update({"ex_accelerator_type": "foo", "ex_accelerator_count": 42})
    call_kwargs = {
        "ex_disk_type": "pd-standard",
        "ex_metadata": {"items": [{"value": None, "key": "salt-cloud-profile"}]},
        "ex_accelerator_count": 42,
        "name": "new",
        "ex_service_accounts": None,
        "external_ip": "ephemeral",
        "ex_accelerator_type": "foo",
        "ex_tags": None,
        "ex_disk_auto_delete": True,
        "ex_network": "default",
        "ex_disks_gce_struct": None,
        "ex_preemptible": False,
        "ex_can_ip_forward": False,
        "ex_on_host_maintenance": "TERMINATE",
        "location": location,
        "ex_subnetwork": None,
        "image": "myimage",
        "size": 1234,
    }

    with patch("salt.cloud.clouds.gce.get_conn", MagicMock(return_value=conn)), patch(
        "salt.cloud.clouds.gce.show_instance", MagicMock()
    ), patch("salt.cloud.clouds.gce.LIBCLOUD_VERSION_INFO", (2, 5, 0)):
        gce.request_instance(config)
        conn.create_node.assert_called_once_with(**call_kwargs)


def test__expand_region():
    """
    Test that _expand_region returns the correct data
    """
    region = DummyGCERegion()
    region.name = "us-central1"

    ret = gce._expand_region(region)
    expected = {"name": "us-central1"}

    assert ret == expected


def test_create_address(conn):
    """
    Test create_address
    """

    region = DummyGCERegion()
    region.name = "us-central1"

    address = DummyGCEAddress()
    address.region = region

    call_args = ("my-ip", region, address)

    with patch("salt.cloud.clouds.gce.get_conn", MagicMock(return_value=conn)), patch(
        "salt.cloud.clouds.gce.LIBCLOUD_VERSION_INFO", (2, 3, 0)
    ):
        kwargs = {"name": "my-ip", "region": region, "address": address}
        gce.create_address(kwargs, "function")
        conn.ex_create_address.assert_called_once_with(*call_args)



# Python libs

import logging

import pytest

# Salt libs
import salt.beacons.network_settings as network_settings
from tests.support.mock import MagicMock, patch

try:
    from pyroute2 import IPDB

    HAS_PYROUTE2 = True
except ImportError:
    HAS_PYROUTE2 = False


log = logging.getLogger(__name__)


class MockIPClass:
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def by_name(self):
        return {}


@pytest.fixture
def configure_loader_modules():
    return {network_settings: {"__context__": {}, "__salt__": {}}}


def test_non_list_config():
    config = {}

    ret = network_settings.validate(config)

    assert ret == (False, "Configuration for network_settings beacon must be a list.")


def test_empty_config():
    config = [{}]

    ret = network_settings.validate(config)

    assert ret == (True, "Valid beacon configuration")


def test_interface():
    config = [{"interfaces": {"enp14s0u1u2": {"promiscuity": None}}}]
    LAST_STATS = network_settings._copy_interfaces_info(
        {"enp14s0u1u2": {"family": "0", "promiscuity": "0", "group": "0"}}
    )

    NEW_STATS = network_settings._copy_interfaces_info(
        {"enp14s0u1u2": {"family": "0", "promiscuity": "1", "group": "0"}}
    )

    ret = network_settings.validate(config)
    assert ret == (True, "Valid beacon configuration")

    with patch.object(network_settings, "LAST_STATS", {}), patch.object(
        network_settings, "IP", MockIPClass
    ), patch(
        "salt.beacons.network_settings._copy_interfaces_info",
        MagicMock(side_effect=[LAST_STATS, NEW_STATS]),
    ):
        ret = network_settings.beacon(config)
        assert ret == []

        ret = network_settings.beacon(config)
        _expected = [
            {
                "interface": "enp14s0u1u2",
                "tag": "enp14s0u1u2",
                "change": {"promiscuity": "1"},
            }
        ]
        assert ret == _expected


def test_interface_no_change():
    config = [{"interfaces": {"enp14s0u1u2": {"promiscuity": None}}}]
    LAST_STATS = network_settings._copy_interfaces_info(
        {"enp14s0u1u2": {"family": "0", "promiscuity": "0", "group": "0"}}
    )

    NEW_STATS = network_settings._copy_interfaces_info(
        {"enp14s0u1u2": {"family": "0", "promiscuity": "0", "group": "0"}}
    )

    ret = network_settings.validate(config)
    assert ret == (True, "Valid beacon configuration")

    with patch.object(network_settings, "LAST_STATS", {}), patch.object(
        network_settings, "IP", MockIPClass
    ), patch(
        "salt.beacons.network_settings._copy_interfaces_info",
        MagicMock(side_effect=[LAST_STATS, NEW_STATS]),
    ):
        ret = network_settings.beacon(config)
        assert ret == []

        ret = network_settings.beacon(config)
        assert ret == []


def test_wildcard_interface():
    config = [{"interfaces": {"en*": {"promiscuity": None}}}]
    LAST_STATS = network_settings._copy_interfaces_info(
        {"enp14s0u1u2": {"family": "0", "promiscuity": "0", "group": "0"}}
    )

    NEW_STATS = network_settings._copy_interfaces_info(
        {"enp14s0u1u2": {"family": "0", "promiscuity": "1", "group": "0"}}
    )

    ret = network_settings.validate(config)
    assert ret == (True, "Valid beacon configuration")

    with patch.object(network_settings, "LAST_STATS", {}), patch.object(
        network_settings, "IP", MockIPClass
    ), patch(
        "salt.beacons.network_settings._copy_interfaces_info",
        MagicMock(side_effect=[LAST_STATS, NEW_STATS]),
    ):
        ret = network_settings.beacon(config)
        assert ret == []

        ret = network_settings.beacon(config)
        _expected = [
            {
                "interface": "enp14s0u1u2",
                "tag": "enp14s0u1u2",
                "change": {"promiscuity": "1"},
            }
        ]
        assert ret == _expected


@pytest.mark.skipif(HAS_PYROUTE2 is False, reason="no pyroute2 installed, skipping")
def test_interface_dict_fields():
    with IPDB() as ipdb:
        for attr in network_settings.ATTRS:
            # ipdb.interfaces is a dict-like object, that
            # contains interface definitions. Interfaces can
            # be referenced both with indices and names.
            #
            # ipdb.interfaces[1] is an interface with index 1,
            # that is the loopback interface.
            assert attr in ipdb.interfaces[1]



"""
Salt states to create and manage VMware vSphere datacenters (datacenters).

:codeauthor: `Alexandru Bleotu <alexandru.bleotu@morganstaley.com>`

Dependencies
============

- pyVmomi Python Module

States
======

datacenter_configured
---------------------

Makes sure a datacenter exists and is correctly configured.

If the state is run by an ``esxdatacenter`` minion, the name of the datacenter
is retrieved from the proxy details, otherwise the datacenter has the same name
as the state.

Supported proxies: esxdatacenter


Example:

1. Make sure that a datacenter named ``target_dc`` exists on the vCenter, using a
``esxdatacenter`` proxy:

Proxy minion configuration (connects passthrough to the vCenter):

.. code-block:: yaml

    proxy:
      proxytype: esxdatacenter
      datacenter: target_dc
      vcenter: vcenter.fake.com
      mechanism: sspi
      domain: fake.com
      principal: host

State configuration:

.. code-block:: yaml

    datacenter_state:
      esxdatacenter.datacenter_configured
"""


import logging

import salt.exceptions

# Get Logging Started
log = logging.getLogger(__name__)
LOGIN_DETAILS = {}


def __virtual__():
    return "esxdatacenter"


def mod_init(low):
    return True


def datacenter_configured(name):
    """
    Makes sure a datacenter exists.

    If the state is run by an ``esxdatacenter`` minion, the name of the
    datacenter is retrieved from the proxy details, otherwise the datacenter
    has the same name as the state.

    Supported proxies: esxdatacenter

    name:
        Datacenter name. Ignored if the proxytype is ``esxdatacenter``.
    """
    proxy_type = __salt__["vsphere.get_proxy_type"]()
    if proxy_type == "esxdatacenter":
        dc_name = __salt__["esxdatacenter.get_details"]()["datacenter"]
    else:
        dc_name = name
    log.info("Running datacenter_configured for datacenter '%s'", dc_name)
    ret = {"name": name, "changes": {}, "result": None, "comment": "Default"}
    comments = []
    si = None
    try:
        si = __salt__["vsphere.get_service_instance_via_proxy"]()
        dcs = __salt__["vsphere.list_datacenters_via_proxy"](
            datacenter_names=[dc_name], service_instance=si
        )
        if not dcs:
            if __opts__["test"]:
                comments.append("State will create datacenter '{}'.".format(dc_name))
            else:
                log.debug("Creating datacenter '%s'", dc_name)
                __salt__["vsphere.create_datacenter"](dc_name, si)
                comments.append("Created datacenter '{}'.".format(dc_name))
            log.info(comments[-1])
            ret["changes"].update({"new": {"name": dc_name}})
        else:
            comments.append(
                "Datacenter '{}' already exists. Nothing to be done.".format(dc_name)
            )
            log.info(comments[-1])
        __salt__["vsphere.disconnect"](si)
        ret["comment"] = "\n".join(comments)
        ret["result"] = None if __opts__["test"] and ret["changes"] else True
        return ret
    except salt.exceptions.CommandExecutionError as exc:
        log.error("Error: %s", exc)
        if si:
            __salt__["vsphere.disconnect"](si)
        ret.update(
            {"result": False if not __opts__["test"] else None, "comment": str(exc)}
        )
        return ret
